<?php
/**
* @package RSMembership!
* @copyright (C) 2009-2014 www.rsjoomla.com
* @license GPL, http://www.gnu.org/licenses/gpl-2.0.html
*/

defined( '_JEXEC' ) or die( 'Restricted access' );
jimport( 'joomla.plugin.plugin');

class plgRSMembershipFlexiContent extends JPlugin
{
	protected $is_installed;

	/*
	* Plugin constructor
	*/
	public function __construct($subject, $config)
	{
		parent::__construct($subject);
		$this->loadLanguage('plg_rsmembership_flexicontent', JPATH_ADMINISTRATOR);
	}

	public function getSupportedSharedTypes()
	{
		return $this->_isFlexiContentInstalled() ? array('flexiarticle', 'flexicategory') : array();
	}

	protected function _isFlexiContentInstalled()
	{
		if (!is_bool($this->is_installed))
		{
			$this->is_installed = file_exists(JPATH_ADMINISTRATOR.'/components/com_flexicontent/manifest.xml');
		}
			
		return $this->is_installed;
	}

	protected function _isSupportedSharedType($shared_type)
	{
		return in_array($shared_type, $this->getSupportedSharedTypes());
	}

	public function showUserFriendlyParams(&$shared)
	{
		if (!$this->_isSupportedSharedType($shared->type))
		{
			return;
		}

		$db 	= JFactory::getDBO();
		$query	= $db->getQuery(true);

		switch ($shared->type)
		{
			case 'flexiarticle':
				$query->select($db->qn('title'))->from($db->qn('#__content'))->where($db->qn('id').' = '.$db->q($shared->params));
				$db->setQuery($query);
				$result = $db->loadResult();

				if ($result)
				{
					$shared->params = $result;
				}
			break;

			case 'flexicategory':
				$query->select($db->qn('title'))->from($db->qn('#__categories'))->where($db->qn('id').' = '.$db->q($shared->params));
				$db->setQuery($query);
				$result = $db->loadResult();

				if ($result)
				{
					$shared->params = $result;
				}
			break;
		}
	}

	public function getData($shared_type, &$data, $limitstart=0, $limit=20)
	{
		if (!$this->_isSupportedSharedType($shared_type))
		{
			return;
		}

		$db 	= JFactory::getDBO();
		$query	= $db->getQuery(true);

		$this->getSortColumn($shared_type, $sortColumn);
		$this->getSortOrder($shared_type, $sortOrder);

		$filter_word = $this->getFilterWord();

		switch ($shared_type)
		{
			case 'flexiarticle':	
				$query->
					select($db->qn('i.title').', '.$db->qn('c.title', 'category_title').', '.$db->qn('i.id'))->
					from($db->qn('#__content', 'i'))->
					join('left', $db->qn('#__categories', 'c').' ON '.$db->qn('c.id').' = '.$db->qn('i.catid'))->
					where($db->qn('i.state').' != '.$db->q('-2'))->
					order($db->qn($sortColumn).' '.$sortOrder);

				if ( $filter_word )
				{
					$query->where($db->qn('i.title').' LIKE '.$db->q('%'.$filter_word.'%'));
				}


				$db->setQuery($query, $limitstart, $limit);
				$data = $db->loadObjectList();
			break;

			case 'flexicategory':
				$params 	= JComponentHelper::getParams('com_flexicontent');
				$component 	= $params->get('flexi_cat_extension', '');

				$query->
					select($db->qn('c.title').', '.$db->qn('c.id'))->
					from($db->qn('#__categories', 'c'))->
					where($db->qn('c.extension').' = '.$db->q($component))->
					order($db->qn($sortColumn).' '.$sortOrder);

				if ( $filter_word )
				{
					$query->where($db->qn('c.title').' LIKE '.$db->q('%'.$filter_word.'%'));
				}

				$db->setQuery($query, $limitstart, $limit);
				$data = $db->loadObjectList();
			break;
		}
	}

	public function getTotal($shared_type, &$total)
	{
		if (!$this->_isSupportedSharedType($shared_type)) return;

		$db 		 = JFactory::getDBO();
		$query		 = $db->getQuery(true);
		$filter_word = $this->getFilterWord();

		switch ($shared_type)
		{
			case 'flexiarticle':
				$query->select('COUNT('.$db->qn('i.id').')')->from($db->qn('#__content', 'i'))->where($db->qn('i.state').' != '.$db->q('-2'));
				
				if ( $filter_word )
				{
					$query->where($db->qn('i.title').' LIKE '.$db->q('%'.$filter_word.'%'));
				}

				$db->setQuery($query);
				$total = $db->loadResult();
			break;

			case 'flexicategory':
				$params 	= JComponentHelper::getParams('com_flexicontent');
				$component 	= $params->get('flexi_cat_extension', '');

				$query->select('COUNT('.$db->qn('c.id').')')->from($db->qn('#__categories', 'c'))->where($db->qn('c.extension').' = '.$db->q($component));

				if ( $filter_word )
				{
					$query->where($db->qn('c.title').' LIKE '.$db->q('%'.$filter_word.'%'));
				}

				$db->setQuery($query);
				$total = $db->loadResult();
			break;
		}
	}

	public function getSortColumn($shared_type, &$sortColumn)
	{
		if (!$this->_isSupportedSharedType($shared_type))
		{
			return;
		}

		$jinput = JFactory::getApplication()->input;

		switch ($shared_type)
		{
			case 'flexiarticle':
				$sortColumn = $jinput->get('filter_order', 'i.ordering', 'string');
			break;

			case 'flexicategory':
				$sortColumn = $jinput->get('filter_order', 'c.lft', 'string');
			break;
		}
	}

	public function getSortOrder($shared_type, &$sortOrder)
	{
		$sortOrder = JFactory::getApplication()->input->get('filter_order_Dir', 'ASC', 'string');
	}

	public function getFilterWord()
	{
		return JFactory::getApplication()->input->get('search', '', 'string');
	}
	
	public function getHeaders($shared_type, &$headers)
	{
		if (!$this->_isSupportedSharedType($shared_type))
		{
			return;
		}
		
		$this->getSortColumn($shared_type, $sortColumn);
		$this->getSortOrder($shared_type, $sortOrder);

		switch ($shared_type)
		{
			case 'flexiarticle':
				$headers = array(			
					array('attributes' => '', 'innerHTML' => JHtml::_('grid.sort', 'TITLE', 'i.title', $sortOrder, $sortColumn)),
					array('attributes' => '', 'innerHTML' => JHtml::_('grid.sort', 'JCATEGORY', 'category_title', $sortOrder, $sortColumn)),
					array('attributes' => 'width="5"', 'innerHTML' => JHtml::_('grid.sort', 'ID', 'i.id', $sortOrder, $sortColumn))
				);
			break;

			case 'flexicategory':
				$headers = array(			
					array('attributes' => '', 'innerHTML' => JHtml::_('grid.sort', 'TITLE', 'c.title', $sortOrder, $sortColumn)),
					array('attributes' => 'width="5"', 'innerHTML' => JHtml::_('grid.sort', 'ID', 'c.id', $sortOrder, $sortColumn))
				);
			break;
		}
	}

	protected function _getItemCategories($id)
	{
		// Get the database object
		$db 	= JFactory::getDBO();
		$query	= $db->getQuery(true);

		$categories = array();

		$query->select($db->qn('catid'))->from($db->qn('#__flexicontent_cats_item_relations'))->where($db->qn('itemid').' = '.$db->q($id));
		$db->setQuery($query);
		$catids = $db->loadColumn();

		foreach ($catids as $catid) 
		{
			$categories = array_merge($categories, $this->_getCategoryParents($catid));
			array_push($categories, $catid);
		}

		return $categories;
	}

	protected function _getCategoryParents($catid)
	{
		$db 		= JFactory::getDBO();
		$query		= $db->getQuery(true);
		$categories = array();

		$query->select($db->qn('parent_id'))->from($db->qn('#__categories'))->where($db->qn('id').' = '.$db->q($catid));
		$db->setQuery($query);

		while ($catid = $db->loadResult())
		{
			$categories[] = $catid;

			$query->clear();
			$query->select($db->qn('parent_id'))->from($db->qn('#__categories'))->where($db->qn('id').' = '.$db->q($catid));
			$db->setQuery($query);
		}

		return $categories;
	}

	public function checkShared($option, &$has_access, &$found_shared, &$redirect, $memberships, $extras)
	{
		$db 	= JFactory::getDBO();
		$query	= $db->getQuery(true);
		$app 	= JFactory::getApplication();
		$jinput = $app->input;
		
		// Runs only in frontend
		if ($app->isAdmin())
		{
			return;
		}
		// Runs only with com_flexicontent
		if ($option != 'com_flexicontent')
		{
			return;
		}
		// Performance check
		if ($has_access)
		{
			return;
		}

		$shared = RSMembershipHelper::getShared($this->getSupportedSharedTypes());
		$view 	= $jinput->get('view', '', 'cmd');
		$id   	= $jinput->get('id', 0, 'int');
		$cid   	= $jinput->get('cid', 0, 'int');

		if ($view == 'item')
		{
			// Must check if it belongs to a parent that we are sharing
			$categories = $this->_getItemCategories($id);

			foreach ($shared as $share)
			{
				$what  = isset($share->membership_id) ? 'membership_id' : 'extra_value_id';
				$where = isset($share->membership_id) ? $memberships : $extras;
				$table = isset($share->membership_id) ? '#__rsmembership_memberships' : '#__rsmembership_extra_values';

				if (($share->type == 'flexiarticle' && $share->params == $id) || ($share->type == 'flexicategory' && in_array($share->params, $categories)))
				{
					$found_shared = true;

					// Found a membership that shares this article
					if (!empty($where) && in_array($share->{$what}, $where))
					{
						$has_access = true;
						break;
					}
					else
					{
						// Get the redirect page
						$query->clear();
						$query->select($db->qn('share_redirect'))->from($db->qn($table))->where($db->qn('id').' = '.$db->q($share->{$what}));
						$db->setQuery($query);
						$redirect = $db->loadResult();
					}
				}
			}
		}
		elseif ($view == 'category' || $view = 'flexicontent') 
		{
			if (!$cid && $jinput->get('Itemid', 0, 'int') > 0) 
			{
				$menu = JFactory::getApplication()->getMenu('site');
				$item = $menu->getItem($jinput->get('Itemid', 0, 'int'));
				$params = new JRegistry($item->params);
				$cid = $params->get('rootcat');
			}
			// Must check if it belongs to a parent that we are sharing
			$categories = $this->_getCategoryParents($cid);
			array_push($categories, $cid);

			foreach ($shared as $share)
			{
				$what  = isset($share->membership_id) ? 'membership_id' : 'extra_value_id';
				$where = isset($share->membership_id) ? $memberships : $extras;
				$table = isset($share->membership_id) ? '#__rsmembership_memberships' : '#__rsmembership_extra_values';

				if ($share->type == 'flexicategory' && in_array($share->params, $categories))
				{
					$found_shared = true;

					// Found a membership that shares this category
					if (!empty($where) && in_array($share->{$what}, $where))
					{
						$has_access = true;
						break;
					}
					else
					{
						// Get the redirect page
						$query->clear();
						$query->select($db->qn('share_redirect'))->from($db->qn($table))->where($db->qn('id').' = '.$db->q($share->{$what}));
						$db->setQuery($query);
						$redirect = $db->loadResult();
					}
				}
			}
		}
	}
}