<?php
/**
* @package RSMembership!
* @copyright (C) 2009-2014 www.rsjoomla.com
* @license GPL, http://www.gnu.org/licenses/gpl-2.0.html
*/

defined( '_JEXEC' ) or die( 'Restricted access' );
jimport( 'joomla.plugin.plugin');

class plgRSMembershipK2 extends JPlugin
{
	protected $is_installed;
	
	/*
	* Plugin constructor
	*/
	public function __construct($subject, $config) {
		parent::__construct($subject);
		$this->loadLanguage('plg_rsmembership_k2', JPATH_ADMINISTRATOR);
	}
	
	public function getSupportedSharedTypes()
	{
		return $this->_isK2Installed() ? array('k2article', 'k2category', 'k2user') : array();
	}
	
	protected function _isK2Installed()
	{
		if ( !is_bool($this->is_installed) )
		{
			$this->is_installed = file_exists(JPATH_ADMINISTRATOR.'/components/com_k2/k2.php');
		}
		return $this->is_installed;
	}

	protected function _isSupportedSharedType($shared_type)
	{
		return in_array($shared_type, $this->getSupportedSharedTypes());
	}
	
	public function showUserFriendlyParams(&$shared)
	{
		if (!$this->_isSupportedSharedType($shared->type))
		{
			return;
		}

		$db 	= JFactory::getDBO();
		$query	= $db->getQuery(true);

		switch ($shared->type)
		{
			case 'k2article':
				$query->select($db->qn('title'))->from($db->qn('#__k2_items'))->where($db->qn('id').' = '.$db->q($shared->params));
				$db->setQuery($query);
				$result = $db->loadResult();
				if ($result)
				{
					$shared->params = $result;
				}
			break;

			case 'k2category':
				$query->select($db->qn('name'))->from($db->qn('#__k2_categories'))->where($db->qn('id').' = '.$db->q( (int) $shared->params ));
				$db->setQuery($query);
				$result = $db->loadResult();

				if ( $result )
				{					
					$shared->params = $result;
				}
			break;

			case 'k2user':
				$query->select($db->qn('username'))->from($db->qn('#__users'))->where($db->qn('id').' = '.$db->q((int) $shared->params));
				$db->setQuery($query);
				$result = $db->loadResult();

				if ( $result )
				{
					$shared->params = $result;
				}
			break;
		}
	}

	public function getData($shared_type, &$data, $limitstart=0, $limit=20)
	{
		if (!$this->_isSupportedSharedType($shared_type)) return;

		$db 	= JFactory::getDBO();
		$query	= $db->getQuery(true);

		$this->getSortColumn($shared_type, $sortColumn);
		$this->getSortOrder($shared_type, $sortOrder);

		$filter_word = $this->getFilterWord();

		switch ( $shared_type ) 
		{
			case 'k2article':
				$query->select($db->qn('i.title').', '.$db->qn('c.name','category_name').', '.$db->qn('i.id'))
					  ->from($db->qn('#__k2_items','i'))
					  ->join('left', $db->qn('#__k2_categories','c').' ON '.$db->qn('c.id').' = '.$db->qn('i.catid'))
					  ->order($db->qn($sortColumn).' '.$sortOrder);
				if ( $filter_word )
				{
					$query->where($db->qn('i.title').' LIKE '.$db->q('%'.$filter_word.'%'));
				}

				$db->setQuery($query, $limitstart, $limit);
				$data = $db->loadObjectList();
			break;

			case 'k2category':
				$query->select($db->qn('c.name').', '.$db->qn('c.id'))->from($db->qn('#__k2_categories', 'c'))->order($db->qn($sortColumn).' '.$sortOrder);

				if ( $filter_word )
				{
					$query->where($db->qn('c.name').' LIKE '.$db->q('%'.$filter_word.'%'));
				}

				$db->setQuery($query, $limitstart, $limit);
				$data = $db->loadObjectList();
			break;

			case 'k2user':
				$query->select($db->qn('username').', '.$db->qn('name').', '.$db->qn('email').', '.$db->qn('id'))->from($db->qn('#__users'))->order($db->qn($sortColumn).' '.$sortOrder);
				if ( $filter_word )
				{
					$query->where(' ( '.$db->qn('name').' LIKE '.$db->q('%'.$filter_word.'%').' OR '.$db->qn('username').' LIKE '.$db->q('%'.$filter_word.'%').' OR '.$db->qn('email').' LIKE '.$db->q('%'.$filter_word.'%').' ) ');
				}
				
				$db->setQuery($query, $limitstart, $limit);
				$data = $db->loadObjectList();
			break;
		}
	}
	
	public function getTotal($shared_type, &$total) 
	{
		if (!$this->_isSupportedSharedType($shared_type)) return;
		
		$db 	= JFactory::getDBO();
		$query	= $db->getQuery(true);

		$filter_word = $this->getFilterWord();

		switch ($shared_type)
		{
			case 'k2article':
			$query->select('COUNT('.$db->qn('i.id').')')
				  ->from($db->qn('#__k2_items','i'));

				if ( $filter_word )
				{
					$query->where($db->qn('i.title').' LIKE '.$db->q('%'.$filter_word.'%'));
				}

				$db->setQuery($query);
				$total = $db->loadResult();
			break;

			case 'k2category':
				$query->select('COUNT('.$db->qn('c.id').')')->from($db->qn('#__k2_categories', 'c'));

				if ( $filter_word )
				{
					$query->where($db->qn('c.name').' LIKE '.$db->q('%'.$filter_word.'%'));
				}

				$db->setQuery($query);
				$total = $db->loadResult();
			break;
			
			case 'k2user':
				$query->select('COUNT('.$db->qn('id').')')->from($db->qn('#__users'));

				if ( $filter_word )
				{
					$query->where(' ( '.$db->qn('name').' LIKE '.$db->q('%'.$filter_word.'%').' OR '.$db->qn('username').' LIKE '.$db->q('%'.$filter_word.'%').' OR '.$db->qn('email').' LIKE '.$db->q('%'.$filter_word.'%').' ) ');
				}

				$db->setQuery($query);
				$total = $db->loadResult();
			break;
		}
	}

	public function getSortColumn($shared_type, &$sortColumn) 
	{
		$jinput = JFactory::getApplication()->input;

		if (!$this->_isSupportedSharedType($shared_type))
		{
			return;
		}

		switch ($shared_type) 
		{
			case 'k2article':
				$sortColumn = $jinput->get('filter_order', 'i.ordering', 'string');
			break;

			case 'k2category':
				$sortColumn = $jinput->get('filter_order', 'c.ordering', 'string');
			break;

			case 'k2user':
				$sortColumn = $jinput->get('filter_order', 'username', 'string');
			break;
		}
	}

	public function getSortOrder($shared_type, &$sortOrder)
	{
		$sortOrder = JFactory::getApplication()->input->get('filter_order_Dir', 'ASC', 'string');
	}

	public function getFilterWord()
	{
		return JFactory::getApplication()->input->get('search', '', 'string');
	}
	
	public function getHeaders($shared_type, &$headers)
	{
		if (!$this->_isSupportedSharedType($shared_type)) return;
		
		$this->getSortColumn($shared_type, $sortColumn);
		$this->getSortOrder($shared_type, $sortOrder);
		
		switch ($shared_type)
		{
			case 'k2article':
				$headers = array(			
					array('attributes' => '', 'innerHTML' => JHtml::_('grid.sort', 'TITLE', 'i.title', $sortOrder, $sortColumn)),
					array('attributes' => '', 'innerHTML' => JHtml::_('grid.sort', 'CATEGORY', 'category_name', $sortOrder, $sortColumn)),
					array('attributes' => 'width="5"', 'innerHTML' => JHtml::_('grid.sort', 'ID', 'i.id', $sortOrder, $sortColumn))
				);
			break;
			
			case 'k2category':
				$headers = array(			
					array('attributes' => '', 'innerHTML' => JHtml::_('grid.sort', 'TITLE', 'c.name', $sortOrder, $sortColumn)),
					array('attributes' => 'width="5"', 'innerHTML' => JHtml::_('grid.sort', 'ID', 'c.id', $sortOrder, $sortColumn))
				);
			break;
			
			case 'k2user':
				$headers = array(			
					array('attributes' => '', 'innerHTML' => JHtml::_('grid.sort', 'USERNAME', 'username', $sortOrder, $sortColumn)),
					array('attributes' => '', 'innerHTML' => JHtml::_('grid.sort', 'NAME', 'name', $sortOrder, $sortColumn)),
					array('attributes' => '', 'innerHTML' => JHtml::_('grid.sort', 'EMAIL', 'email', $sortOrder, $sortColumn)),
					array('attributes' => 'width="5"', 'innerHTML' => JHtml::_('grid.sort', 'ID', 'a.id', $sortOrder, $sortColumn))
				);
			break;
		}
	}
	
	protected function _getItemCategories($id)
	{
		// Get the database object
		$db 	= JFactory::getDBO();
		$query	= $db->getQuery(true);

		$query->select($db->qn('catid'))->from($db->qn('#__k2_items'))->where($db->qn('id').' = '.$db->q($id));
		$db->setQuery($query);

		$catid 		= $db->loadResult();
		$categories = $this->_getCategoryParents($catid);
		array_push($categories, $catid);
		
		return $categories;
	}

	protected function _getCategoryParents($catid) 
	{
		$db 	= JFactory::getDBO();
		$query	= $db->getQuery(true);

		$categories = array();
		$query->select($db->qn('parent'))->from($db->qn('#__k2_categories'))->where($db->qn('id').' = '.$db->q($catid));
		$db->setQuery($query);

		while ($catid = $db->loadResult())
		{
			$categories[] = $catid;

			$query->clear();
			$query->select($db->qn('parent'))->from($db->qn('#__k2_categories'))->where($db->qn('id').' = '.$db->q($catid));
			$db->setQuery($query);
		}

		return $categories;
	}
	
	public function checkShared($option, &$has_access, &$found_shared, &$redirect, $memberships, $extras)
	{	
		$app = JFactory::getApplication();
		
		// Runs only in frontend
		if ($app->isAdmin())
		{
			return;
		}
		// Runs only with com_k2
		if ($option != 'com_k2')
		{
			return;
		}
		// Performance check
		if ($has_access)
		{
			return;
		}
		
		// Get the database object
		$db 	= JFactory::getDBO();
		$query	= $db->getQuery(true);
		
		$shared = RSMembershipHelper::getShared($this->getSupportedSharedTypes());
		
		$jinput = $app->input;
		$view 	= $jinput->get('view', '', 'cmd');
		$task 	= $jinput->get('task', '', 'cmd');
		$layout = $jinput->get('layout', '', 'cmd');
		$id 	= $jinput->get('id', '', 'int');

		if ( $view == 'item' )
		{
			// Must check if it belongs to a parent that we are sharing
			$categories = $this->_getItemCategories($id);
			
			foreach ($shared as $share)
			{
				$what  = isset($share->membership_id) ? 'membership_id' : 'extra_value_id';
				$where = isset($share->membership_id) ? $memberships : $extras;
				$table = isset($share->membership_id) ? '#__rsmembership_memberships' : '#__rsmembership_extra_values';
					
				if (($share->type == 'k2article' && $share->params == $id) || ($share->type == 'k2category' && in_array($share->params, $categories)))
				{
					$found_shared = true;

					// Found a membership that shares this article
					if (!empty($where) && in_array($share->{$what}, $where))
					{
						$has_access = true;
						break;
					}
					else
					{
						// Get the redirect page
						$query->clear();
						$query->select($db->qn('share_redirect'))->from($db->qn($table))->where($db->qn('id').' = '.$db->q($share->{$what}));
						$db->setQuery($query);
						$redirect = $db->loadResult();
					}
				}
			}
		}
		elseif ( $view == 'itemlist' ) 
		{
			if ($task == 'category' || $layout == 'category')
			{
				if (!$id && $jinput->get('Itemid', 0, 'int') > 0)
				{
					$menu = $app->getMenu('site');
					$item = $menu->getItem($jinput->get('Itemid', 0, 'int'));
					$params = new JRegistry($item->params);
					$id = $params->get('categories');
				}
				// Must check if it belongs to a parent that we are sharing
				$categories = array();
				if (is_array($id))
					foreach ($id as $catid)
					{
						$categories = array_merge($categories, $this->_getCategoryParents($catid));
						array_push($categories, $catid);
					}
				else
				{
					$categories = $this->_getCategoryParents($id);
					array_push($categories, $id);
				}
				
				foreach ($shared as $share)
				{
					$what  = isset($share->membership_id) ? 'membership_id' : 'extra_value_id';
					$where = isset($share->membership_id) ? $memberships : $extras;
					$table = isset($share->membership_id) ? '#__rsmembership_memberships' : '#__rsmembership_extra_values';
						
					if ($share->type == 'k2category' && in_array($share->params, $categories))
					{
						$found_shared = true;
						
						// Found a membership that shares this category
						if (!empty($where) && in_array($share->{$what}, $where))
						{
							$has_access = true;
							break;
						}
						else
						{
							// Get the redirect page
							$query->clear();
							$query->select($db->qn('share_redirect'))->from($db->qn($table))->where($db->qn('id').' = '.$db->q($share->{$what}));
							$db->setQuery($query);
							$redirect = $db->loadResult();
						}
					}
				}
			}
			if ( $task == 'user' )
			{
				foreach ($shared as $share)
				{
					$what  = isset($share->membership_id) ? 'membership_id' : 'extra_value_id';
					$where = isset($share->membership_id) ? $memberships : $extras;
					$table = isset($share->membership_id) ? '#__rsmembership_memberships' : '#__rsmembership_extra_values';
						
					if ($share->type == 'k2user' && $share->params == $id) 
					{
						$found_shared = true;
						
						// Found a membership that shares this category
						if (!empty($where) && in_array($share->{$what}, $where))
						{
							$has_access = true;
							break;
						}
						else
						{
							// Get the redirect page
							$query->clear();
							$query->select($db->qn('share_redirect'))->from($db->qn($table))->where($db->qn('id').' = '.$db->q($share->{$what}));
							$db->setQuery($query);
							$redirect = $db->loadResult();
						}
					}
				}
			}
		}
	}
}