// Make the plugin independent
if (typeof RSMembership == 'undefined') {
	var RSMembership = {};

	RSMembership.parseJSON = function (data) {
		if (typeof data != 'object') {
			var match = data.match(/{.*}/);
			return jQuery.parseJSON(match[0]);
		}
		return jQuery.parseJSON(data);
	};
}

if (typeof RSMembership.utils == 'undefined') {
	RSMembership.utils = {};
}

RSMembership.utils.compare = function(compare, operator, value) {
	switch (operator) {
		case '>':   return compare > value;
		case '<':   return compare < value;
		case '>=':  return compare >= value;
		case '<=':  return compare <= value;
		case '==':  return compare == value;
		case '!=':  return compare != value;
		case '===': return compare === value;
		case '!==': return compare !== value;
	}
};

RSMembership.authorize = {
	check_card: function(what) {
		what.value = what.value.replace(/[^0-9]+/g, '');
	},

	check_year: function(what) {
		if (what.value.length == 2) {
			what.value = '20' + what.value;
		}

		var currDate = new Date();

		if (what.value < currDate.getFullYear()) {
			what.value = currDate.getFullYear();
		}
	},

	check_authorize: function(url) {
		var has_error = false;

		var checks = {
			rsm_csc_number	: {operator: '<' , compare: 3},
			rsm_cc_exp_yy	: {operator: '!=' , compare: 4},
			rsm_cc_fname	: {operator: '==' , compare: 0},
			rsm_cc_lname	: {operator: '==' , compare: 0}
		};

		jQuery.each(checks, function(element, operation) {
			element = jQuery('#'+element);
			if (RSMembership.utils.compare(element.val().length, operation.operator, operation.compare)) {
				if (!element.hasClass('rsm_error')){
					element.addClass("rsm_error");
				}
				has_error = true;
			} else {
				element.removeClass('rsm_error');
			}
		});
		
		var action_url  = jQuery('#action_url').val();
		var response 	= jQuery('#rsm_response');

		if (has_error) {
			response.html(rsm_get_error_message(0));
		} else {
			// All form data
			var cc_number 	= jQuery('#rsm_cc_number');
			var csc_number 	= jQuery('#rsm_csc_number');
			var cc_exp_mm  	= jQuery('#rsm_cc_exp_mm');
			var cc_exp_yy  	= jQuery('#rsm_cc_exp_yy');
			var cc_fname   	= jQuery('#rsm_cc_fname');
			var cc_lname   	= jQuery('#rsm_cc_lname');

			var params = {
				cc_number 		: cc_number.val(),
				csc_number 		: csc_number.val(),
				cc_exp_mm 		: cc_exp_mm.val(),
				cc_exp_yy 		: cc_exp_yy.val(),
				cc_fname 		: encodeURIComponent(cc_fname.val()),
				cc_lname 		: encodeURIComponent(cc_lname.val()),
				membership_id 	: jQuery('#membership_id').val()
			};

			jQuery.ajax({
				type    	: 'POST',
				url     	: url,
				dataType  	: 'html',
				data      	: params,
				beforeSend	: function () {
					jQuery('#rsm_pay_button').css('visibility', 'hidden');
					jQuery('#rsm_loading').css('display', 'block');
				}
			}).done(function (responseText) {
				jQuery('#rsm_pay_button').css('visibility', 'visible');
				jQuery('#rsm_loading').css('display', 'none');

				if (responseText.indexOf('RSM_AUTHORIZE_OK') > -1)
				{
					window.location.href = action_url;
				}
				else if (responseText.indexOf('RSM_SESSION_END') > -1)
				{
					window.location.href = action_url;
				}
				else
				{
					response.html(responseText);
				}
			});
		}
	}
};

// initiate the only special tooltip from this form
var rsm_use_legacy_tooltip = false;
jQuery(function(){
	var rsm_tooltip_content = jQuery('#rsm_tooltip').html();
	if (jQuery.fn.tooltip == 'undefined') {
		rsm_use_legacy_tooltip = true;
	} else {
		jQuery('#rsm_whats_csc').tooltip({
			html: true,
			title: rsm_tooltip_content
		});
	}
});


// Legacy functions
function rsm_check_card(what)
{
	RSMembership.authorize.check_card(what);
}

function rsm_check_year(what)
{
	RSMembership.authorize.check_year(what);
}

function rsm_check_authorize(url)
{
	RSMembership.authorize.check_authorize(url);
}

var rsm_tooltip=function(){
	var id = 'rsm_tt';
	var top = 3;
	var left = 3;
	var maxw = 400;
	var speed = 10;
	var timer = 20;
	var endalpha = 95;
	var alpha = 0;
	var tt,t,c,b,h;
	var ie = document.all ? true : false;
	return{
		show:function(v,w){
			if (!rsm_use_legacy_tooltip) {
				return;
			}

			if(tt == null){
				tt = document.createElement('div');
				tt.setAttribute('id',id);
				t = document.createElement('div');
				t.setAttribute('id',id + 'top');
				c = document.createElement('div');
				c.setAttribute('id',id + 'cont');
				b = document.createElement('div');
				b.setAttribute('id',id + 'bot');
				tt.appendChild(t);
				tt.appendChild(c);
				tt.appendChild(b);
				document.body.appendChild(tt);
				tt.style.opacity = 0;
				tt.style.filter = 'alpha(opacity=0)';
				document.onmousemove = this.pos;
			}
			tt.style.display = 'block';
			c.innerHTML = document.getElementById(v).innerHTML;
			tt.style.width = w ? w + 'px' : 'auto';
			if(!w && ie){
				t.style.display = 'none';
				b.style.display = 'none';
				tt.style.width = tt.offsetWidth;
				t.style.display = 'block';
				b.style.display = 'block';
			}
			if(tt.offsetWidth > maxw){tt.style.width = maxw + 'px'}
			h = parseInt(tt.offsetHeight) + top;
			clearInterval(tt.timer);
			tt.timer = setInterval(function(){rsm_tooltip.fade(1)},timer);
		},
		pos:function(e){
			var u = ie ? event.clientY + document.documentElement.scrollTop : e.pageY;
			var l = ie ? event.clientX + document.documentElement.scrollLeft : e.pageX;
			tt.style.top = (u - h) + 'px';
			tt.style.left = (l + left) + 'px';
		},
		fade:function(d){
			var a = alpha;
			if((a != endalpha && d == 1) || (a != 0 && d == -1)){
				var i = speed;
				if(endalpha - a < speed && d == 1){
					i = endalpha - a;
				}else if(alpha < speed && d == -1){
					i = a;
				}
				alpha = a + (i * d);
				tt.style.opacity = alpha * .01;
				tt.style.filter = 'alpha(opacity=' + alpha + ')';
			}else{
				clearInterval(tt.timer);
				if(d == -1){tt.style.display = 'none'}
			}
		},
		hide:function(){
			if (!rsm_use_legacy_tooltip) {
				return;
			}

			clearInterval(tt.timer);
			tt.timer = setInterval(function(){rsm_tooltip.fade(-1)},timer);
		}
	};
}();