<?php
/**
* @package RSMembership!
* @copyright (C) 2009-2015 www.rsjoomla.com
* @license GPL, http://www.gnu.org/licenses/gpl-2.0.html
*/

defined('_JEXEC') or die('Restricted access');

class plgSystemRSMembershipAuthorize extends JPlugin
{
	public function __construct(&$subject, $config) {
		parent::__construct($subject, $config);
		if ($this->canRun() && $this->params)
		{
			$this->_loadLanguage();
			$name = $this->params->get('payment_name', 'Credit Card');
			$tax_details = $this->getTaxDetails('rsmembershipauthorize');
			RSMembership::addPlugin($this->getTranslation($name), 'rsmembershipauthorize', $tax_details);
		}
	}
	
	protected function getTranslation($text) {
		$lang = JFactory::getLanguage();
		$key  = str_replace(' ', '_', $text);
		if ($lang->hasKey($key))
		{
			return JText::_($key);
		}
		else 
		{
			return $text;
		}
	}

	protected function canRun() {
		static $checked = false;
		
		if (!class_exists('RSMembershipHelper') && file_exists(JPATH_ADMINISTRATOR.'/components/com_rsmembership/helpers/rsmembership.php')) 
		{
			require_once JPATH_ADMINISTRATOR.'/components/com_rsmembership/helpers/rsmembership.php';
		}
		
		return class_exists('RSMembershipHelper') && class_exists('RSMembership');
	}
	
	protected function getTaxDetails($plugin) {
		$tax_value = $this->params->get('tax_value');
		if (!empty($tax_value))
		{
			return array('tax_type' => $this->params->get('tax_type'), 'tax_value' => $tax_value);
		}
		return false;
	}
	
	public function onMembershipCancelPayment($plugin, $data, $membership, &$transaction) 
	{
		if (!$this->canRun()) 
		{
			return;
		}
	
		if ($plugin != 'rsmembershipauthorize')
		{
			return false;
		}
		
		if (!$membership->recurring || $membership->period == 0)
		{
			return false;
		}

		$authorize = RSMembershipAuthorizeNet::getInstance($this->params);
		$data = array(
			'subscription_id' => $transaction->custom
		);

		$response = $authorize->handleSubscription('cancelSubscriptionRequest', $data);

		$log = array();
		$log[] = 'Cancelled by request. Response is below:';
		$log[] = '--- START ---';
		$log[] = 'Ref Id: '.($response->approved ? $response->authorize_raw_response->getRefId(): '');
		$log[] = 'Result Code: '.($response->approved ? $response->authorize_raw_response->getMessages()->getResultCode() : '');
		$log[] = 'Code: '.$response->response_reason_code;
		$log[] = 'Text: '.$response->response_reason_text;
		$log[] = 'Subscription Id: '.$data['subscription_id'];
		$log[] = '--- END ---';

		RSMembership::saveTransactionLog($log, $transaction->id);

		if ($response->approved) {
			return true;
		}

		JFactory::getApplication()->enqueueMessage($response->response_reason_text, 'warning');
		return false;
	}
	
	protected function showControl($label, $input, $help='') {
		$html = '<div class="control-group">'."\n".
					"\t".'<div class="control-label">'."\n".
						"\t\t".$label."\n".
					"\t".'</div>'."\n".
					"\t".'<div class="controls">'."\n".
						"\t\t".$input."\n".
						$help.
					"\t".'</div>'."\n".
				'</div>'."\n";
		return $html;
	}
	
	public function onMembershipPayment($plugin, $data, $extra, $membership, $transaction, $html) {
		if (!$this->canRun()) 
		{
			return;
		}
		
		if ($plugin != 'rsmembershipauthorize')
		{
			return false;
		}

		// Generate invoice number
		$this->_setInvoiceNumber();

		// Load the bootstrap tooltip
		JHtml::_('bootstrap.tooltip');

		JHtml::_('script', 'plg_system_rsmembershipauthorize/script.js', array('relative' => true, 'version'=> 'auto'));
		JHtml::_('stylesheet', 'plg_system_rsmembershipauthorize/style.css', array('relative' => true, 'version' => 'auto'));


		$fields = $this->_getFields();
		
		$transaction->gateway = 'Authorize.Net';
		
		$tax_value = $this->_getTax($transaction->price);
		// store the tax details
		if (!empty($tax_value))
		{
			$transaction->tax_value = $tax_value;

			$tax_details = $this->getTaxDetails($plugin);
			$transaction->tax_type = $tax_details['tax_type'];
		}

		$actionUrl 		= JRoute::_('index.php?option=com_rsmembership&task=thankyou');
		$authorizeUrl 	= JUri::root(true).'/index.php?option=com_rsmembership&plugin_task=authorize';
		$payMessage		= JText::sprintf('PLG_SYSTEM_RSMEMBERSHIPAUTHORIZE_AUTHORIZE_PRICE_DESC', RSMembershipHelper::getPriceFormat($transaction->price + $tax_value));
		$html = '<div class="form-horizontal">'."\n".
				'<fieldset>'."\n".
					'<legend>'.JText::_('PLG_SYSTEM_RSMEMBERSHIPAUTHORIZE_AUTHORIZE_CARD_INFO').'</legend>'."\n".
					$this->showControl($fields['cc_number'][0], $fields['cc_number'][1], '<span class="help-inline">'.JHtml::image('plg_system_rsmembershipauthorize/cc_logos.gif', '', null, true).'</span>'."\n").
					$this->showControl($fields['csc_number'][0], $fields['csc_number'][1], '<span class="help-inline" id="rsm_whats_csc" onmouseover="rsm_tooltip.show(\'rsm_tooltip\');" onmouseout="rsm_tooltip.hide();">'.JText::_('PLG_SYSTEM_RSMEMBERSHIPAUTHORIZE_AUTHORIZE_WHATS_CSC').'</span>'."\n").
					$this->showControl($fields['cc_exp_mm'][0], $fields['cc_exp_mm'][1]).
					$this->showControl($fields['cc_exp_yy'][0], $fields['cc_exp_yy'][1]).
					$this->showControl($fields['cc_fname'][0], $fields['cc_fname'][1]).
					$this->showControl($fields['cc_lname'][0], $fields['cc_lname'][1]).
					'<p>'.$payMessage.'</p>'."\n".
					'<div id="rsm_response" class="rsm_response_error"></div>'."\n".
					'<div class="form-actions">'."\n".
						'<button class="button btn btn-large btn-success" type="button" onclick="rsm_check_authorize(\''.addslashes($authorizeUrl).'\');" id="rsm_pay_button">'.JText::_('PLG_SYSTEM_RSMEMBERSHIPAUTHORIZE_AUTHORIZE_PAY_NOW').'</button>'.JHtml::image('com_rsmembership/load.gif', '', 'id="rsm_loading" style="display: none;"', true)."\n".
					'</div>'."\n".
				'</fieldset>'."\n".
				'<input type="hidden" id="membership_id" value="'.$membership->id.'" />'."\n".
				'<input type="hidden" id="action_url" value="'.$actionUrl.'" />'."\n".
				'</div>'."\n";
		
		// Tooltip
		$html .= '<div id="rsm_tooltip" style="display: none;">'."\n".
					'<div>'.JText::_('PLG_SYSTEM_RSMEMBERSHIPAUTHORIZE_AUTHORIZE_WHATS_CSC_DESC').'</div>'."\n".
					'<div align="center">'.JHtml::image('plg_system_rsmembershipauthorize/cc_csc.gif', 'CSC', null, true).'</div>'."\n".
				 '</div>'."\n";
		
		$warning = JHtml::image('plg_system_rsmembershipauthorize/warning.png', '', 'id="rsm_warning"', true);
		
		// Script
		$html .= '<script type="text/javascript">'."\n".
				 'function rsm_get_error_message(code) {'."\n".
				 "if (code == 0) return '".$warning."<br/> ".JText::_('PLG_SYSTEM_RSMEMBERSHIPAUTHORIZE_AUTHORIZE_ERRORS', true)."';\n".
				 '}'."\n".
				 '</script>'."\n";
				 
		return $html;
	}
	
	protected function _getFields() {
		$fields = array();

		$field = new stdClass();
		$field->id 	       = '';
		$field->name       = 'cc_number';
		$field->label      = JText::_('PLG_SYSTEM_RSMEMBERSHIPAUTHORIZE_AUTHORIZE_CC_NUMBER');
		$field->type       = 'textbox';
		$field->values     = '';
		$field->additional = 'maxlength="16" autocomplete="off"'; // onkeydown="return rsm_check_card(this);" onkeyup="return rsm_check_card(this);"
		$field->required   = 1;
		$fields['cc_number'] = RSMembershipHelper::showCustomField($field, array(), true, false);
		
		$field = new stdClass();
		$field->id 	       = '';
		$field->name       = 'csc_number';
		$field->label      = JText::_('PLG_SYSTEM_RSMEMBERSHIPAUTHORIZE_AUTHORIZE_CSC');
		$field->type       = 'textbox';
		$field->values     = '';
		$field->additional = 'style="width: 45px; text-align: center;" maxlength="4" autocomplete="off"'; // onkeydown="return rsm_check_card(this);" onkeyup="return rsm_check_card(this);"
		$field->required   = 1;
		$fields['csc_number'] = RSMembershipHelper::showCustomField($field, array(), true, false);
		
		$field = new stdClass();
		$field->id 	       = '';
		$field->name       = 'cc_image';
		$field->label      = '';
		$field->type       = 'freetext';
		$field->values     = JHtml::image('plg_system_rsmembershipauthorize/cc_logos.gif', 'Credit Cards', null, true);
		$field->additional = '';
		$field->required   = 0;
		$fields['cc_image'] = RSMembershipHelper::showCustomField($field, array(), true, false);

		$field = new stdClass();
		$field->id 	       = '';
		$field->name       = 'cc_exp_mm';
		$field->label      = JText::_('PLG_SYSTEM_RSMEMBERSHIPAUTHORIZE_MONTH');
		$field->type       = 'select';
		$field->values     = array();
		
		for ($i=1; $i<=12; $i++)
		{	
			$field->values[] = ($i < 10 ? '0'.$i : $i).'-'.JText::_('PLG_SYSTEM_RSMEMBERSHIPAUTHORIZE_AUTHORIZE_MONTH_'.$i);
		}	
		
		$field->values 	   = implode("\n", $field->values);
		$field->additional = '';
		$field->required   = 1;
		$fields['cc_exp_mm'] = RSMembershipHelper::showCustomField($field, array(), true, false);
		
		$field = new stdClass();
		$field->id 	       = '';
		$field->name       = 'cc_exp_yy';
		$field->label      = JText::_('PLG_SYSTEM_RSMEMBERSHIPAUTHORIZE_YEAR');
		$field->type       = 'textbox';
		$field->values     = '';
		$field->additional = 'style="width: 35px; text-align: center;" maxlength="4" onblur="rsm_check_year(this)" autocomplete="off"'; // onkeydown="return rsm_check_card(this);" onkeyup="return rsm_check_card(this);"
		$field->required   = 1;
		$fields['cc_exp_yy'] = RSMembershipHelper::showCustomField($field, array(), true, false);
		
		$field = new stdClass();
		$field->id 	       = '';
		$field->name       = 'cc_fname';
		$field->label      = JText::_('PLG_SYSTEM_RSMEMBERSHIPAUTHORIZE_AUTHORIZE_FIRST_NAME');
		$field->type       = 'textbox';
		$field->values     = '';
		$field->additional = 'maxlength="64" autocomplete="off"';
		$field->required   = 1;
		$fields['cc_fname'] = RSMembershipHelper::showCustomField($field, array(), true, false);
		
		$field = new stdClass();
		$field->id 	       = '';
		$field->name       = 'cc_lname';
		$field->label      = JText::_('PLG_SYSTEM_RSMEMBERSHIPAUTHORIZE_AUTHORIZE_LAST_NAME');
		$field->type       = 'textbox';
		$field->values     = '';
		$field->additional = 'maxlength="64" autocomplete="off"';
		$field->required   = 1;
		$fields['cc_lname'] = RSMembershipHelper::showCustomField($field, array(), true, false);
		
		return $fields;
	}
	
	protected function _getTax($price) {
		$tax_value = $this->params->get('tax_value');
		if (!empty($tax_value))
		{
			$tax_type = $this->params->get('tax_type');
			
			// percent ?
			if ($tax_type == 0)
				$tax_value = $price * ($tax_value / 100);
		}

		return $tax_value;
	}

	public function hasDelayTransactionStoring() {
		return true;
	}

	public function delayTransactionStoring($vars) {
		if ($vars['plugin'] == 'rsmembershipauthorize')
		{
			$properties 	= $vars['properties'];
			
			if (!empty($properties['price']))
			{
				$vars['delay'] 	= true;
				
				JFactory::getSession()->set('transaction', $properties, 'rsmembership');
				return true;
			}
		}
	}

	protected function getDelayedTransaction() {
		return JFactory::getSession()->get('transaction', null, 'rsmembership');
	}

	protected function emptyDelayedTransaction() {
		JFactory::getSession()->set('transaction', null, 'rsmembership');
	}
	
	public function getLimitations() {
		return JText::_('PLG_SYSTEM_RSMEMBERSHIPAUTHORIZE_AUTHORIZE_LIMITATIONS');
	}
	
	protected function onPaymentNotification() {
		if (!$this->canRun())
		{	
			return;
		}
		
		$jinput = JFactory::getApplication()->input;
		
		$subscription_id = $jinput->get('x_subscription_id', 0, 'int');
		if ($subscription_id)
		{
			$db 	= JFactory::getDBO();
			$query	= $db->getQuery(true);

			$response_code = $jinput->get('x_response_code', '', 'int');
			$reason_code   = $jinput->get('x_response_reason_code', '', 'int');

			$query
				->select('*')
				->from($db->qn('#__rsmembership_transactions'))
				->where($db->qn('gateway').' = '.$db->q('Authorize.Net'))
				->where($db->qn('custom').' = '.$db->q($subscription_id));
			$db->setQuery($query);
			$transaction = $db->loadObject();
			if (!$transaction)
			{
				return;
			}

			$log = array();
			if ($response_code == 1)
			{
				$query->clear();
				$query
					->select($db->qn('id'))
					->select($db->qn('membership_id'))
					->from($db->qn('#__rsmembership_membership_subscribers'))
					->where($db->qn('from_transaction_id').' = '.$db->q($transaction->id));
				$db->setQuery($query);
				$membership = $db->loadObject();

				$query->clear()
					->select($db->qn('activation'))
					->from($db->qn('#__rsmembership_memberships'))
					->where($db->qn('id').' = '.$db->q((int) $membership->membership_id));
				$db->setQuery($query);
				$activation = $db->loadResult();

				JTable::addIncludePath(JPATH_ADMINISTRATOR.'/components/com_rsmembership/tables');
				$row 	= JTable::getInstance('Transaction','RSMembershipTable');
				$date 	= JFactory::getDate();

				$row->user_id 	 = $transaction->user_id;
				$row->user_email = $transaction->user_email;
				$row->user_data	 = $transaction->user_data;
				$row->type 		 = 'renew';
				$params = array();
				$params[] = 'id='.$membership->id;
				$params[] = 'membership_id='.$membership->membership_id;
				$row->params 	 = implode(';', $params); // params, membership, extras etc
				$row->date 		 = $date->toSql();
				$row->ip 		 = JFactory::getApplication()->input->server->getString('REMOTE_ADDR');
				$row->price 	 = $jinput->get('x_amount', '', 'string');
				$row->currency 	 = RSMembershipHelper::getConfig('currency');
				$row->hash 		 = $jinput->get('x_trans_id', '', 'string');
				$row->gateway 	 = 'Authorize.Net';
				$row->status 	 = 'pending';
				// store the transaction
				$row->store();

				RSMembership::finalize($row->id);
				if ($activation != MEMBERSHIP_ACTIVATION_MANUAL)
				{
					RSMembership::approve($row->id);
				}
			} 
			elseif ($response_code == 2)
			{
				// declined
				$log[] = 'Payment has been declined - Response Code: '.$response_code.' Reason code: '.$reason_code;
			}
			elseif ($response_code == 3 && $reason_code == 8)
			{
				// expired card
				$log[] = 'Credit card has expired - Response Code: '.$response_code.' Reason code: '.$reason_code;
			}
			else
			{
				// other ?
				$log[] = 'Other - Response Code: '.$response_code.' Reason code: '.$reason_code;
			}

			if ($log)
			{
				RSMembership::saveTransactionLog($log, $transaction->id);
			}
		}
	}
	
	protected function escape($string) {
		return htmlspecialchars($string, ENT_COMPAT, 'utf-8');
	}
	
	protected function addLog($entry) {
		// Add timestamp to the entry
		$entry = JFactory::getDate()->format('[Y-m-d H:i:s]').' - '.$entry."\n";
		
		// Compute the log file's path.
		static $path;
		if (!$path)
		{
			$config = new JConfig();
			$path 	= $config->log_path.'/rsmembership-authorize.php';
			
			if (!file_exists($path))
			{
				file_put_contents($path, "<?php die('Forbidden.'); ?>\n\n");
			}
		}
		
		if ($this->params->get('enable_logging')) 
		{
			file_put_contents($path, $entry, FILE_APPEND);
		}
	}
	
	protected function _authorize($data) {
		extract($data);
		$input		   = JFactory::getApplication()->input;
		$cc_number 	   = $input->get('cc_number', '', 'cmd');
		$csc_number	   = $input->get('csc_number', '', 'string');
		$cc_expiration = substr($input->get('cc_exp_mm', '', 'cmd'), 0, 2).'-'.$input->get('cc_exp_yy', 0, 'int');
		$cc_fname	   = $input->get('cc_fname', '', 'string');
		$cc_lname	   = $input->get('cc_lname', '', 'string');

		$user_data 	 = !empty($row->user_data) ? (object) unserialize($row->user_data) : (object) array();
		$user_fields = is_object($user_data) && isset($user_data->fields) ? $user_data->fields : array();
		
		$address 	= $this->_getMappedField('x_address', $user_fields);
		$state 		= $this->_getMappedField('x_state', $user_fields);
		$city 		= $this->_getMappedField('x_city', $user_fields);

		$country 	= $this->_getMappedField('x_country', $user_fields);
		$zipcode 	= $this->_getMappedField('x_zip', $user_fields);
		$phone 		= $this->_getMappedField('x_phone', $user_fields);
		$fax 		= $this->_getMappedField('x_fax', $user_fields);
		$company 	= $this->_getMappedField('x_company', $user_fields);

		$authorize = RSMembershipAuthorizeNet::getInstance($this->params);

		$data = array(
			'amount' 			 	=> $row->price,
			'card_num' 			 	=> $cc_number,
			'exp_date' 				=> $cc_expiration,
			'card_code' 		 	=> $csc_number,
			'invoice_num' 		 	=> RSMembershipHelper::getConfig('currency'),
			'description' 			=> $description,
			 //transaction_id
			'first_name' 		 	=> $cc_fname,
			'ship_to_first_name' 	=> $cc_fname,
			"last_name"			 	=> $cc_lname,
			"ship_to_last_name"	 	=> $cc_lname,
			"email"				 	=> $row->get('user_email'),
			"address"				=> !empty($address) ? $address : '',
			"ship_to_address"		=> !empty($address) ? $address : '',
			"state"					=> !empty($state) ? $state : '',
			"ship_to_state"			=> !empty($state) ? $state : '',
			"city"					=> !empty($city) ? $city : '',
			"ship_to_city"			=> !empty($city) ? $city : '',
			"country"				=> !empty($country) ? $country : '',
			"ship_to_country"		=> !empty($country) ? $country : '',
			"zip"					=> !empty($zipcode) ? $zipcode : '',
			"ship_to_zip"			=> !empty($zipcode) ? $zipcode : '',
			"phone"					=> !empty($phone) ? $phone : '',
			"fax"					=> !empty($fax) ? $fax : '',
			"company"				=> !empty($company) ? $company : '',
			"ship_to_company"		=> !empty($company) ? $company : '',
		);

		$response = $authorize->makeTransaction('authOnlyTransaction', $data);

		// Add the response to the log
		$this->addLog('Response is:');
		$this->addLog(print_r($response, true));

		if ($response->approved)
		{
			return $response;
		}
		else
		{
			$image = !empty($response->response_reason_code) ? 'warning' : 'error';
			throw new Exception(JHtml::image('plg_system_rsmembershipauthorize/'.$image.'.png', 'Information', array('id' => 'rsm_warning'), true).' '.$this->escape($response->response_reason_text));
		}
	}
	
	protected function _capture($auth_trans_id) {
		$authorize = RSMembershipAuthorizeNet::getInstance($this->params);

		$data = array(
			'transaction_id' => $auth_trans_id
		);

		$response = $authorize->makeTransaction('priorAuthCaptureTransaction', $data);

		// Add the response to the log
		$this->addLog('Response is:');
		$this->addLog(print_r($response, true));

		if ($response->approved)
		{
			return $response;
		}
		else
		{
			$image = !empty($response->response_reason_code) ? 'warning' : 'error';
			throw new Exception(JHtml::image('plg_system_rsmembershipauthorize/'.$image.'.png', 'Information', array('id' => 'rsm_warning'), true).' '.$this->escape($response->response_reason_text));
		}
	}
	
	protected function _void($auth_trans_id) {

		$authorize = RSMembershipAuthorizeNet::getInstance($this->params);

		$data = array(
			'transaction_id' => $auth_trans_id
		);

		$response = $authorize->makeTransaction('voidTransaction', $data);

		// Add the response to the log
		$this->addLog('Response is:');
		$this->addLog(print_r($response, true));

		if ($response->approved)
		{
			return $response;
		}
		else
		{
			$image = !empty($response->response_reason_code) ? 'warning' : 'error';
			throw new Exception(JHtml::image('plg_system_rsmembershipauthorize/'.$image.'.png', 'Information', array('id' => 'rsm_warning'), true).' '.$this->escape($response->response_reason_text));
		}
	}
	
	protected function _subscribe($data) {
		extract($data);
		list($length, $unit) 	= $this->_getAuthorizeLength($membership);
		$invoiceNumber 			= $this->_getInvoiceNumber();
		$startDate 				= JFactory::getDate()->modify("+$length $unit");
		$params 				= RSMembershipHelper::parseParams($row->params);
		$input		   			= JFactory::getApplication()->input;
		$db						= JFactory::getDbo();
		$query					= $db->getQuery(true);
		
		// Calculate extra total
		$extra_total = 0;
		if (!empty($params['extras']))
		{
			$extras_q = '';
			foreach ($params['extras'] as $k => $id)
			{
				$extras_q[] = $db->q($id);
			}

			$query->clear();
			$query
				->select('SUM('.$db->qn('price').')')
				->from($db->qn('#__rsmembership_extra_values'))
				->where($db->qn('id').' IN ('.implode(',', $extras_q).')');
			$db->setQuery($query);
			$extra_total = $db->loadResult();
		}

		// Calculate amount to be paid
		$amount  = $membership->use_renewal_price ? $membership->renewal_price : $membership->price;
		$amount	+= $extra_total;
		$amount += $this->_getTax($amount);

		// Are we using trials?
		$trialOccurrences = $membership->use_trial_period ? 1 : 0;
		$trialAmount 	  = $membership->use_trial_period ? $membership->trial_price : 0;
		$trialAmount	 += $extra_total;
		$trialAmount	 += $this->_getTax($trialAmount);
		
		// Number of recurring times
		$totalOccurences  = ($membership->recurring_times > 0 ? $membership->recurring_times : '9999');

		$cc_number 	   	= $input->get('cc_number', '', 'cmd');
		$csc_number	   	= $input->get('csc_number', '', 'string');
		$cc_expiration 	= substr($input->get('cc_exp_mm', '', 'cmd'), 0, 2).'-'.$input->get('cc_exp_yy', 0, 'int');
		$cc_fname	   	= $input->get('cc_fname', '', 'string');
		$cc_lname	   	= $input->get('cc_lname', '', 'string');
		
		$user_data 	 = !empty($row->user_data) ? (object) unserialize($row->user_data) : (object) array();
		$user_fields = is_object($user_data) && isset($user_data->fields) ? $user_data->fields : array();
		
		$address 	= $this->_getMappedField('x_address', $user_fields);
		$state 		= $this->_getMappedField('x_state', $user_fields);
		$city 		= $this->_getMappedField('x_city', $user_fields);
		$country 	= $this->_getMappedField('x_country', $user_fields);
		$zipcode 	= $this->_getMappedField('x_zip', $user_fields);
		$phone 		= $this->_getMappedField('x_phone', $user_fields);
		$fax 		= $this->_getMappedField('x_fax', $user_fields);
		$company 	= $this->_getMappedField('x_company', $user_fields);
		$email		= $row->user_email;

		$authorize = RSMembershipAuthorizeNet::getInstance($this->params);

		$data = array(
			'amount' 			 	=> $amount,
			'trial_amount' 			=> $trialAmount,
			'card_num' 			 	=> $cc_number,
			'exp_date' 				=> $cc_expiration,
			'card_code' 		 	=> $csc_number,

			'invoice_num' 		 	=> $invoiceNumber,
			'description' 			=> $description,

			'unit'					=> $unit,
			'length'				=> $length,

			'start_date'			=> $startDate,
			'total_occurrences'		=> $totalOccurences,
			'trial_occurrences'		=> $trialOccurrences,

			'email'					=> $email,
			'customer_phone'		=> !empty($phone) ? $phone : '',
			'customer_fax'			=> !empty($fax) ? $fax : '',

			'first_name' 		 	=> $cc_fname,
			"last_name"			 	=> $cc_lname,
			"company"				=> !empty($company) ? $company : '',
			"address"				=> !empty($address) ? $address : '',
			"city"					=> !empty($city) ? $city : '',
			"state"					=> !empty($state) ? $state : '',
			"zip"					=> !empty($zipcode) ? $zipcode : '',
			"country"				=> !empty($country) ? $country : '',
		);



		$post_url = $this->params->get('mode') ? net\authorize\api\constants\ANetEnvironment::PRODUCTION : net\authorize\api\constants\ANetEnvironment::SANDBOX;
		$this->addLog('ARB Request: POSTing to '.$post_url);
		$this->addLog('POSTing value...');

		$response = $authorize->handleSubscription('createSubscriptionRequest', $data);
		// format the date before logging
		$data['start_date'] = $data['start_date']->format('Y-m-d', true);
		$this->addLog(print_r($data, true));


		$this->addLog('Response is: ');
		$this->addLog(print_r($response, true));


		if ($response->approved)
		{
			return $response->subscription_id;
		}
		else
		{
			$image = !empty($response->response_reason_code) ? 'warning' : 'error';
			throw new Exception(JHtml::image('plg_system_rsmembershipauthorize/'.$image.'.png', 'Information', array('id' => 'rsm_warning'), true).' '.$this->escape($response->response_reason_text));
		}
	}

	public function onAfterRoute() {
		// Get application
		$app = JFactory::getApplication();
		
		// Do not run in the administrator section
		if ($app->isClient('administrator'))
		{
			return true;
		}
		
		// Get jinput for covenience
		$jinput	= $app->input;

		// Is this a recurring payment notification?
		if ( $jinput->get('authorizepayment', '', 'string') )
		{
			return $this->onPaymentNotification();
		}

		// Get option and task
		$option = $jinput->get('option', '', 'string');
		$task   = $jinput->get('plugin_task', '', 'cmd');

		// Is this an Authorize.NET request?
		if ($option == 'com_rsmembership' && $task == 'authorize')
		{
			if (ob_get_contents())
			{
				ob_end_clean();
			}

			$db 	= JFactory::getDBO();
			$query	= $db->getQuery(true);
			$membership_id = $jinput->get('membership_id', 0, 'int');

			$query
				->select('*')
				->from($db->qn('#__rsmembership_memberships'))
				->where($db->qn('id').' = '.$db->q($membership_id));
			$db->setQuery($query);
			$membership = $db->loadObject();

			JTable::addIncludePath(JPATH_ADMINISTRATOR.'/components/com_rsmembership/tables');
			$row		 = JTable::getInstance('Transaction', 'RSMembershipTable');
			$transaction = $this->getDelayedTransaction();

			// Need to check if the transaction is still in the session
			if (empty($transaction))
			{
				$app->enqueueMessage(JText::_('PLG_SYSTEM_RSMEMBERSHIPAUTHORIZE_SESSION_EXPIRED'), 'error');
				echo 'RSM_SESSION_END';
				die();
			}
			// Need to check that cURL is enabled
			if (!function_exists('curl_init'))
			{
				echo JHtml::image('plg_system_rsmembershipauthorize/error.png', 'Error', array('id' => 'rsm_warning'), true).' '.JText::_('PLG_SYSTEM_RSMEMBERSHIPAUTHORIZE_AUTHORIZE_CURL_ERROR');
				die();
			}
			$row->bind($transaction);

			// adjust price
			$row->price += $this->_getTax($row->price);
			$row->price  = $this->_convertNumber($row->price);

			$description  = $this->params->get('message_type') ? $membership->name : JText::sprintf('PLG_SYSTEM_RSMEMBERSHIPAUTHORIZE_MEMBERSHIP_PURCHASE_ON', RSMembershipHelper::showDate($row->date));
			$is_recurring = $membership->recurring && $membership->period > 0 && $row->type == 'new';
			
			$log = array();

			try
			{
				// If it's recurring, make sure credit card is valid on the first renewal term
				if ($is_recurring)
				{
					$cc_expiration_month = substr($jinput->get('cc_exp_mm', '', 'cmd'), 0, 2);
					$cc_expiration_year = $jinput->get('cc_exp_yy', 0, 'int');

					$last = JFactory::getDate($cc_expiration_year . '-' . $cc_expiration_month . '-15 00:00:00');
					$last = JFactory::getDate($cc_expiration_year . '-' . $cc_expiration_month . '-' . $last->format('t') . ' 23:59:59');
					
					list($length, $unit) = $this->_getAuthorizeLength($membership);
					$startDate = JFactory::getDate()->modify("+$length $unit");
					
					if ($startDate->toUnix() > $last->toUnix())
					{
						throw new Exception(JText::sprintf('PLG_SYSTEM_RSMEMBERSHIPAUTHORIZE_CREDIT_CARD_EXPIRES_BEFORE_START', RSMembershipHelper::showDate($startDate->toSql())));
					}
				}
				
				// Authorization - this is the only way to make sure credit card details are correct before creating the ARB subscription
				$response 		= $this->_authorize(array(
					'row' 			=> $row,
					'description' 	=> $description
				));
				$auth_trans_id 	= $response->transaction_id;
				
				// If it's recurring, make sure you can setup a recurring payment
				if ($is_recurring)
				{
					$subscriptionId = $this->_subscribe(array(
						'row' 			=> $row,
						'membership' 	=> $membership,
						'description'	=> $description
					));
					
					$row->custom = $subscriptionId;
				}
				
				// Capture previously authorized payment (ie actual pay happens here)
				$response = $this->_capture($auth_trans_id);

				// Everything is ok, store the order number and the transaction
				$row->hash = $response->transaction_id;
				$row->store();

				$nice_log = 'Response from gateway is: ';
				if (!empty($response->nice_response)) {
					foreach ($response->nice_response as $key => $message) {
						$key = str_replace('_', '', $key);
						$nice_log .= "\n".ucfirst($key).' => '.$message;
					}
				}
				$log[] = $nice_log;

				$activationText = 'missing';
				if ($membership->activation == MEMBERSHIP_ACTIVATION_MANUAL)
				{
					$activationText = 'manual';
				}
				elseif ($membership->activation == MEMBERSHIP_ACTIVATION_AUTO)
				{
					$activationText = 'auto';
				}
				elseif ($membership->activation == MEMBERSHIP_ACTIVATION_INSTANT)
				{
					$activationText = 'instant';
				}

				$log[] = "Activation is $activationText.";
				
				// Time to remove the transaction from the session
				$this->emptyDelayedTransaction();

				RSMembership::finalize($row->get('id'));

				if ($membership->activation != MEMBERSHIP_ACTIVATION_MANUAL)
				{
					RSMembership::approve($row->get('id'));
				}

				RSMembership::saveTransactionLog($log, $row->id);

				echo 'RSM_AUTHORIZE_OK';
			}
			catch (Exception $e)
			{
				if (!empty($auth_trans_id))
				{
					try
					{
						$response = $this->_void($auth_trans_id);
					}
					catch (Exception $voidException)
					{
						echo $voidException->getMessage().';';
					}
				}
				echo $e->getMessage();
				die();
			}
			
			die();
		}
	}
	
	protected function _getMappedField($field, $user_fields) {
		$return = '';
		
		if ($name = $this->params->get($field))
		{
			if (isset($user_fields[$name]))
			{
				$return = is_array($user_fields[$name]) ? implode(', ', $user_fields[$name]) : $user_fields[$name];
			}
		}
		
		return $return;
	}
	
	protected function _getAuthorizeLength($membership) {
		$length = $membership->period;
		$unit 	= '';
		
		switch ($membership->period_type)
		{
			case 'h':
				$length = 7;
				$unit 	= 'days';
			break;
			
			case 'd':
				if ($membership->period > 365)
				{
					$length = 365;
				}
				
				$unit = 'days';
			break;
			
			case 'm':
				if ($membership->period > 12)
				{
					$length = 12;
				}
				
				$unit = 'months';
			break;
			
			case 'y':
				if ($membership->period >= 1)
				{
					$length = 365;
				}
					
				$unit = 'days';
			break;
		}
		
		return array($length, $unit);
	}
	
	protected function _setInvoiceNumber() {
		JFactory::getSession()->set('com_rsmembership.invoice', substr(md5(uniqid('invoice')), 0, 20));
	}
	
	protected function _getInvoiceNumber() {
		return JFactory::getSession()->get('com_rsmembership.invoice');
	}
	
	protected function _convertNumber($number) {
		return number_format($number, 2, '.', '');
	}
	
	protected function _convertPeriod($period, $type) {
		$converted = array(
			$period,
			strtoupper($type)
		);

		return $converted;
	}
	
	protected function _loadLanguage() {
		$this->loadLanguage('plg_system_rsmembership');
		$this->loadLanguage('plg_system_rsmembershipauthorize');
	}
}


/**
 * Class RSFormProAuthorizeNet
 */

use net\authorize\api\contract\v1 as AnetAPI;
use net\authorize\api\controller as AnetController;

class RSMembershipAuthorizeNet
{
	/**
	 * hold the environment type (Sandbox or Live)
	 */
	private $environment;

	/**
	 * hold the authentication object
	 */
	private $merchantAuthentication;

	/**
	 * holds the option to send or not an email by authorize upon subscription
	 */
	private $email_customer;

	/**
	 * @return RSMembershipAuthorizeNet
	 */
	public static function getInstance($params)
	{
		static $inst;
		if (!$inst)
		{
			$inst = new RSMembershipAuthorizeNet($params);
		}

		return $inst;
	}

	/**
	 * RSFormProAuthorizeNet constructor.
	 */
	public function __construct($params)
	{
		require_once JPATH_ADMINISTRATOR . '/components/com_rsmembership/helpers/authorize/autoload.php';

		$this->merchantAuthentication = new AnetAPI\MerchantAuthenticationType();
		$this->merchantAuthentication->setName($params->get('x_login'));
		$this->merchantAuthentication->setTransactionKey($params->get('x_tran_key'));

		// Set the environment type
		$this->environment = net\authorize\api\constants\ANetEnvironment::PRODUCTION;
		$mode = (bool) $params->get('mode', '0');
		if (!$mode)
		{
			$this->environment = net\authorize\api\constants\ANetEnvironment::SANDBOX;
		}

		$this->email_customer = (bool) $params->get('email_customer');
	}

	// Transactions
	public function makeTransaction($transactionType = '', $vars = array()) {
		if (empty($transactionType) || !in_array($transactionType, array('authOnlyTransaction', 'priorAuthCaptureTransaction', 'voidTransaction'))) {
			return (object)array('approved' => false, 'response_reason_text' => 'The transaction is not available!');
		}

		if (empty($vars)) {
			return (object)array('approved' => false, 'response_reason_text' => 'No arguments are specified for this transaction!');
		}

		try {
			// if the transaction_id is available then the request is either a capture or void
			$isTransaction_id = !empty($vars['transaction_id']) ? $vars['transaction_id'] : false;

			if (!$isTransaction_id) {
				// Create the payment data for a credit card
				$creditCard = new AnetAPI\CreditCardType();
				$creditCard->setCardNumber($vars['card_num']);
				$creditCard->setExpirationDate($vars['exp_date']);

				if (isset($vars['card_code'])) {
					$creditCard->setCardCode($vars['card_code']);
				}

				// Add the payment data to a paymentType object
				$paymentOne = new AnetAPI\PaymentType();
				$paymentOne->setCreditCard($creditCard);

				$customerAddressFields = array(
					'first_name' => 'setFirstName',
					'last_name' => 'setLastName',
					'company' => 'setCompany',
					'address' => 'setAddress',
					'city' => 'setCity',
					'state' => 'setState',
					'zip' => 'setZip',
					'country' => 'setCountry',
					'phone' => 'setPhoneNumber',
					'fax' => 'setFaxNumber',
					'email' => 'setEmail',
				);

				// Set the customer's Bill To address
				$customerAddress = false;
				foreach ($customerAddressFields as $cField => $function) {
					if (isset($vars[$cField])) {
						if (!$customerAddress) {
							$customerAddress = new AnetAPI\CustomerAddressType();
						}

						$customerAddress->{$function}($vars[$cField]);
					}
				}

				// Set the customer's Ship To address, if specified
				$customerShipAddress = false;
				unset($customerAddressFields['email'], $customerAddressFields['phone'], $customerAddressFields['fax']);
				foreach ($customerAddressFields as $cField => $function) {
					if (isset($vars['ship_to_' . $cField])) {
						if (!$customerShipAddress) {
							$customerShipAddress = new AnetAPI\CustomerAddressType();
						}

						$customerShipAddress->{$function}($vars['ship_to_' . $cField]);
					}
				}

				// Add values for transaction settings
				$duplicateWindowSetting = new AnetAPI\SettingType();
				$duplicateWindowSetting->setSettingName("duplicateWindow");
				$duplicateWindowSetting->setSettingValue("60");

				// Don't send the email to the customer
				$email_customer = new AnetAPI\SettingType();
				$email_customer->setSettingName("emailCustomer");

				if (!$this->email_customer) {
					$email_customer->setSettingValue("0");
				} else {
					$email_customer->setSettingValue("1");
				}

				// Create order information
				$order_info = array(
					'invoice_num' => 'setInvoiceNumber',
					'description' => 'setDescription',
				);
				$order = false;
				foreach ($order_info as $oldVar => $function) {
					if (!empty($vars[$oldVar])) {
						if (!$order) {
							$order = new AnetAPI\OrderType();
						}

						$order->{$function}($vars[$oldVar]);
					}
				}

				// CustomerDataType
				$customerData_info = array(
					'email' => 'setEmail',
					'customer_type' => 'setType',
					'customer_id' => 'setId',
				);
				$customerData = false;
				foreach ($customerData_info as $oldVar => $function) {
					if (!empty($vars[$oldVar])) {
						if (!$customerData) {
							$customerData = new AnetAPI\CustomerDataType();
						}

						$customerData->{$function}($vars[$oldVar]);
					}
				}
			}

			// Create a TransactionRequestType object and add the previous objects to it
			$transactionRequestType = new AnetAPI\TransactionRequestType();
			$transactionRequestType->setTransactionType($transactionType);

			if (!$isTransaction_id) {
				$transactionRequestType->setAmount($vars['amount']);
				if ($order) {
					$transactionRequestType->setOrder($order);
				}
				$transactionRequestType->setPayment($paymentOne);

				if ($customerAddress) {
					$transactionRequestType->setBillTo($customerAddress);
				}

				if ($customerShipAddress) {
					$transactionRequestType->setShipTo($customerShipAddress);
				}

				// set the customerIP
				$customerIP = isset($vars['customer_ip']) ? $vars['customer_ip'] : '';

				if (empty($customerIP)) {
					$jinput = JFactory::getApplication()->input;

					$customerIP = $jinput->server->get('REMOTE_ADDR', '');
				}

				if (!empty($customerIP) && $this->testIp($customerIP)) {
					$transactionRequestType->setCustomerIP($customerIP);
				}

				// Set the Customer Data Type
				if ($customerData) {
					$transactionRequestType->setCustomer($customerData);
				}

				$transactionRequestType->addToTransactionSettings($duplicateWindowSetting);
				$transactionRequestType->addToTransactionSettings($email_customer);
			} else {
				$transactionRequestType->setRefTransId($isTransaction_id);
			}

			// Set the transaction's refId
			$refId = 'ref' . time();

			$request = new AnetAPI\CreateTransactionRequest();
			$request->setMerchantAuthentication($this->merchantAuthentication);

			if ($transactionType != 'priorAuthCaptureTransaction') {
				$request->setRefId($refId);
			}


			// Add to the request the transaction request type
			$request->setTransactionRequest($transactionRequestType);

			// Create the controller and get the response
			$controller = new AnetController\CreateTransactionController($request);

			return $this->parseResponseTransaction($controller->executeWithApiResponse($this->environment), $transactionType);
		} catch (Exception $e) {
			return (object)array('approved' => false, 'response_reason_text' => $e->getMessage());
		}
	}


	// Subscriptions
	public function handleSubscription($subscriptionType = '', $vars = array()) {
		if (empty($subscriptionType) || !in_array($subscriptionType, array('createSubscriptionRequest', 'cancelSubscriptionRequest'))) {
			return (object)array('approved' => false, 'response_reason_text' => 'The subscription request is not available!');
		}

		if (empty($vars)) {
			return (object)array('approved' => false, 'response_reason_text' => 'No arguments are specified for this subscription!');
		}

		try {
			// if the subscription_id is available then the request is a subscription cancellation
			$isSubscription_id = !empty($vars['subscription_id']) ? $vars['subscription_id'] : false;

			$subscription = false;
			if (!$isSubscription_id) {
				// Subscription Type Info
				$subscription = new AnetAPI\ARBSubscriptionType();
				$subscription->setName($vars['description']);

				$interval = new AnetAPI\PaymentScheduleType\IntervalAType();
				$interval->setLength($vars['length']);
				$interval->setUnit($vars['unit']);

				$paymentSchedule = new AnetAPI\PaymentScheduleType();
				$paymentSchedule->setInterval($interval);
				$paymentSchedule->setStartDate($vars['start_date']);
				$paymentSchedule->setTotalOccurrences($vars['total_occurrences']);
				$paymentSchedule->setTrialOccurrences($vars['trial_occurrences']);

				$subscription->setPaymentSchedule($paymentSchedule);
				$subscription->setAmount($vars['amount']);
				$subscription->setTrialAmount($vars['trial_amount']);

				// Create the payment data for a credit card
				$creditCard = new AnetAPI\CreditCardType();
				$creditCard->setCardNumber($vars['card_num']);
				$creditCard->setExpirationDate($vars['exp_date']);

				if (isset($vars['card_code'])) {
					$creditCard->setCardCode($vars['card_code']);
				}

				// Add the payment data to a paymentType object
				$payment = new AnetAPI\PaymentType();
				$payment->setCreditCard($creditCard);
				$subscription->setPayment($payment);

				$order = new AnetAPI\OrderType();
				$order->setInvoiceNumber($vars['invoice_num']);
				$subscription->setOrder($order);

				$customerAddressFields = array(
					'first_name' => 'setFirstName',
					'last_name' => 'setLastName',
					'company' => 'setCompany',
					'address' => 'setAddress',
					'city' => 'setCity',
					'state' => 'setState',
					'zip' => 'setZip',
					'country' => 'setCountry'
				);

				// Set the customer's Bill To address
				$billTo = false;
				foreach ($customerAddressFields as $cField => $function) {
					if (isset($vars[$cField])) {
						if (!$billTo) {
							$billTo = new AnetAPI\NameAndAddressType();
						}

						$billTo->{$function}($vars[$cField]);
					}
				}

				if ($billTo) {
					// bill to
					$subscription->setBillTo($billTo);
					// ship to is the same as bill to
					$subscription->setShipTo($billTo);
				}

				// CustomerDataType
				$customerData_info = array(
					'email' => 'setEmail',
					'customer_type' => 'setType',
					'customer_id' => 'setId',
					'customer_phone' => 'setPhoneNumber',
					'customer_fax' => 'setFaxNumber',
				);
				$customerData = false;
				foreach ($customerData_info as $oldVar => $function) {
					if (!empty($vars[$oldVar])) {
						if (!$customerData) {
							$customerData = new AnetAPI\CustomerType();
						}

						$customerData->{$function}($vars[$oldVar]);
					}
				}

				if ($customerData) {
					$subscription->setCustomer($customerData);
				}

			}

			// Set the transaction's refId
			$refId = 'ref' . time();

			$request = $isSubscription_id ? new AnetAPI\ARBCancelSubscriptionRequest() : new AnetAPI\ARBCreateSubscriptionRequest();
			$request->setMerchantAuthentication($this->merchantAuthentication);
			$request->setRefId($refId);

			if ($subscription) {
				$request->setSubscription($subscription);
			}

			if ($isSubscription_id) {
				$request->setSubscriptionId($isSubscription_id);
			}

			// Create the controller and get the response
			$controller =  $isSubscription_id ? new AnetController\ARBCancelSubscriptionController($request) : new AnetController\ARBCreateSubscriptionController($request);

			return $this->parseResponseSubscription($controller->executeWithApiResponse($this->environment), !$isSubscription_id);
		} catch (Exception $e) {
			return (object)array('approved' => false, 'response_reason_text' => $e->getMessage());
		}
	}


	protected function testIp($ip) {
		return filter_var($ip, FILTER_VALIDATE_IP, FILTER_FLAG_IPV4);
	}

	protected function parseResponseTransaction($response, $transactionType) {
		$result = (object) array(
			'approved' => false,
			'response_reason_text' => '',
			'response_reason_code' => '',
			'transaction_id' => '',
			'authorize_raw_response' => $response,
			'nice_response' => array()
		);

		if (is_null($response)) {
			$result->response_reason_text = 'No response returned';

			return $result;
		}

		// get the response details
		$response_messages = $response->getMessages();
		$transaction_messages = $response->getTransactionResponse();


		if ($response_messages->getResultCode() == "Ok") {
			if ( $transaction_messages != null && $transaction_messages->getMessages() != null) {
				$result->approved = true;
				if ($transactionType == 'priorAuthCaptureTransaction') {
					$result->transaction_id = $transaction_messages->getRefTransId();
				} else {
					$result->transaction_id = $transaction_messages->getTransId();
				}

				$nice_response = array(
					'transaction_id' => $result->transaction_id,
					'transaction_response_code' =>  $transaction_messages->getResponseCode(),
					'message_code' =>  $transaction_messages->getMessages()[0]->getCode(),
					'auth_code' =>  $transaction_messages->getAuthCode(),
					'description' =>  $transaction_messages->getMessages()[0]->getDescription(),
				);

				$result->nice_response = $nice_response;
			}  else {
				$result->response_reason_text = 'Transaction Failed';
				if ($transaction_messages->getErrors() != null) {
					$transaction_errors =  $transaction_messages->getErrors()[0];
					$result->response_reason_code =  $transaction_errors->getErrorCode();
					$result->response_reason_text .=  "\n". $transaction_errors->getErrorText();
				}
			}
		} else {
			$result->response_reason_text = 'Transaction Failed';

			if ($transaction_messages != null && $transaction_messages->getErrors() != null) {
				$transaction_errors =  $transaction_messages->getErrors()[0];
				// Return the message for the transaction error
				$result->response_reason_code =  $transaction_errors->getErrorCode();
				$result->response_reason_text .=  "\n". $transaction_errors->getErrorText();
			} else {
				if ($response_messages != null) {
					// If no transaction error then return the api error messages
					$result->response_reason_code = $response_messages->getMessage()[0]->getCode();
					$result->response_reason_text .= "\n". $response_messages->getMessage()[0]->getText();
				}
			}
		}

		if (!$result->approved) {
			$nice_response = array(
				'error_code' => $result->response_reason_code,
				'error_message' =>  $result->response_reason_text,
			);

			$result->nice_response = $nice_response;
		}

		return $result;
	}

	protected function parseResponseSubscription($response, $return_id = false) {
		$result = (object) array(
			'approved' => false,
			'response_reason_text' => '',
			'response_reason_code' => '',
			'subscription_id' => '',
			'authorize_raw_response' => $response
		);

		if (is_null($response)) {
			$result->response_reason_text = 'No response returned';

			return $result;
		}

		// get the response details
		$response_messages = $response->getMessages();

		if ($response_messages->getResultCode() == "Ok") {
			$result->approved = true;
			// if it is a subscription must return the subscription ID
			if ($return_id) {
				$result->subscription_id = $response->getSubscriptionId();
			}

		}

		$result->response_reason_code = $response_messages->getMessage()[0]->getCode();
		$result->response_reason_text = $response_messages->getMessage()[0]->getText();

		return $result;
	}
}