<?php
/**
* @package RSMembership!
* @copyright (C) 2017-2018 www.minijoomla.com
* @license GPL, http://www.gnu.org/licenses/gpl-2.0.html
*/

defined('_JEXEC') or die('Restricted access');

if (file_exists(JPATH_ADMINISTRATOR.'/components/com_rsmembership/helpers/rsmembership.php')) {
	require_once JPATH_ADMINISTRATOR.'/components/com_rsmembership/helpers/rsmembership.php';
}

class plgSystemRSMembershipSpicePay extends JPlugin
{
	public function __construct(&$subject, $config)
	{
		parent::__construct($subject, $config);

		if ($this->canRun() && $this->params)
		{
			$this->_loadLanguage();
			$name = $this->params->get('payment_name', 'SpicePay');

			if (JFactory::getLanguage()->getTag() != 'fa-IR')
    		{
    			RSMembership::addPlugin($this->getTranslation($name), 'rsmembershipspicepay');
    		}
		}
	}

	protected function getTranslation($text)
	{
		$lang = JFactory::getLanguage();
		$key  = str_replace(' ', '_', $text);
		if ($lang->hasKey($key))
		{
			return JText::_($key);
		}
		else
		{
			return $text;
		}
	}

	protected function canRun()
	{
		return file_exists(JPATH_ADMINISTRATOR.'/components/com_rsmembership/helpers/rsmembership.php');
	}

	public function onMembershipPayment($plugin, $data, $extra, $membership, $transaction, $html)
	{
		if (!$this->canRun()) return;
		if ($plugin != 'rsmembershipspicepay') return false;

		$db 	= JFactory::getDBO();
		$query	= $db->getQuery(true);

		$transaction->price += $this->_getTax($transaction->price);

		$transaction->custom = md5($transaction->params.' '.time());

		if ($membership->activation == 2)
		{
			$transaction->status = 'completed';
		}

		//$transaction->store();

		$url = 'https://www.spicepay.com/pay.php';

		$html = '<p>'.JText::_('PLG_SYSTEM_RSMEMBERSHIPSPICEPAY_PLEASE_WAIT_REDIRECT').'</p>';
		$html .= '<form method="post" action="'.$url.'" id="spicepayForm">';
		
		if ($this->params->get('siteId'))
		{
			$html .= '<input type="hidden" name="siteId" value="'.$this->escape($this->params->get('siteId')).'" />';
		}
		else
		{
			$html .= '<input type="hidden" name="merchantId" value="'.$this->escape($this->params->get('merchantId')).'" />';
		}

		$html .= '<input type="hidden" name="language" value="'.$this->escape($this->params->get('language')).'" />';

		$html .= '<input type="hidden" name="'.$this->escape($this->params->get('currency')).'" value="'.$this->_convertNumber($transaction->price).'" />';

		$html .= '<input type="hidden" name="orderId" value="'.$this->escape($transaction->custom).'" />';
		$html .= '<input type="hidden" name="clientId" value="'.JFactory::getUser()->id.'" />';
		$html .= '<input type="hidden" name="custom1" value="'.$this->escape($transaction->custom).'" />';

		$html .= '</form>';

		$html .= '<script type="text/javascript">';
		$html .= 'function spicepayFormSubmit() { window.setTimeout(function() { document.getElementById(\'spicepayForm\').submit() }, 5500); }';
		$html .= 'try { window.addEventListener ? window.addEventListener("load",spicepayFormSubmit,false) : window.attachEvent("onload",spicepayFormSubmit); }';
		$html .= 'catch (err) { spicepayFormSubmit(); }';
		$html .= '</script>';

		return $html;
	}

	protected function _getTax($price)
	{
		$tax_value = $this->params->get('tax_value');

		if (!empty($tax_value))
		{
			$tax_type = $this->params->get('tax_type');

			// percent ?
			if ($tax_type == 0)
				$tax_value = $price * ($tax_value / 100);
		}

		return $tax_value;
	}
	
	public function onAfterRender()
	{
		$app = JFactory::getApplication();		
		if($app->getName() != 'site') return;

		$spicepaypayment = $app->input->get('spicepaypayment', '', 'string');

		if (!empty($spicepaypayment))
			$this->onPaymentNotification();
	}

	public function getLimitations()
	{
		return JText::_('PLG_SYSTEM_RSMEMBERSHIPSPICEPAY_SPICEPAY_LIMITATIONS');
	}

	protected function addLog($entry)
	{
		// Add timestamp to the entry
		$entry = JFactory::getDate()->format('[Y-m-d H:i:s]').' - '.$entry."\n";

		// Compute the log file's path.
		static $path;
		if (!$path) {
			$config = new JConfig();
			$path 	= $config->log_path.'/rsmembership-spicepay.php';
			
			if (!file_exists($path)) {
				file_put_contents($path, "<?php die('Forbidden.'); ?>\n\n");
			}
		}

		if ($this->params->get('enable_logging')) {
			file_put_contents($path, $entry, FILE_APPEND);
		}

		echo $entry;
	}

	protected function finish()
	{
		JFactory::getApplication()->close();
	}
	
	protected function onPaymentNotification()
	{
		if (!$this->canRun()) return;

		ob_end_clean();

		$name = $this->getTranslation($this->params->get('payment_name', 'SpicePay'));

		require_once JPATH_ADMINISTRATOR.'/components/com_rsmembership/helpers/adapters/input.php';

		$db 	= JFactory::getDBO();
		$query	= $db->getQuery(true);
		$jinput = RSInput::create();
		$log = array();

		// assign posted variables to local variables
		$paymentId			= $jinput->get('paymentId', '', 'none');
		$orderId			= $jinput->get('orderId', '', 'none');
		$clientId			= $jinput->get('clientId', '', 'none');
		$paymentAmountBTC	= $jinput->get('paymentAmountBTC', '', 'none');
		$paymentAmountUSD 	= $jinput->get('paymentAmountUSD', '', 'none');
		$paymentAmountEUR 	= $jinput->get('paymentAmountEUR', '', 'none');
		$receivedAmountBTC	= $jinput->get('receivedAmountBTC', '', 'none');
		$receivedAmountUSD	= $jinput->get('receivedAmountUSD', '', 'none');
		$receivedAmountEUR	= $jinput->get('receivedAmountEUR', '', 'none');
		$hash				= $jinput->get('hash', '', 'none');
		$status 			= $jinput->get('status', '', 'none');
		$custom1 			= $jinput->get('orderId', 0, 'none');

		$secret_code		= $this->params->get('secret_code');

		// try to get the transaction id based on the custom hash
		$transaction_id = $this->getTransactionId($custom1);

		// Do not deny the transaction for now.
		$deny = false;
		
		$this->addLog("Transaction ID is '$transaction_id', based on '$custom1'.");

		if ($this->params->get('currency') == 'amountUSD')
		{
			$paymentAmount   = $paymentAmountUSD;
			$receivedAmount  = $receivedAmountUSD;
			$paymentCurrency = 'amountUSD';
		}
		else
		{
			$paymentAmount   = $paymentAmountEUR;
			$receivedAmount  = $receivedAmountEUR;
			$paymentCurrency = 'amountEUR';
		}

		if (strcmp(md5($secret_code . $paymentId . $custom1 . $clientId . $paymentAmountBTC . $paymentAmount . $receivedAmountBTC . $receivedAmount . $status), $hash) == 0)
		{
			$log[] = "Payment status is ".(!empty($status) ? $status : 'empty').".";

			// check that hash has not been previously processed
			// check custom_hash from db -> if custom_hash == custom1
			$query->clear();
			$query
				->select($db->qn('id'))
				->from($db->qn('#__rsmembership_transactions'))
				->where($db->qn('hash').' = '.$db->q($hash))
				->where($db->qn('gateway').' = '.$db->q($name));
			$db->setQuery($query);

			if (!$db->loadResult()) 
			{
				$query->clear();
				$query
					->select('*')
					->from($db->qn('#__rsmembership_transactions'))
					->where($db->qn('custom').' = '.$db->q($custom1))
					->where($db->qn('status').' != '.$db->q('completed'));
				$db->setQuery($query);
				$transaction = $db->loadObject();

				// check if transaction exists
				if (empty($transaction))
				{
					$log[] = "Could not identify transaction with custom hash $custom1. Stopping.";
				}
			}
			else
			{
				$log[] = "The transaction $hash has already been processed. Stopping.";
			}
			
			if (!empty($transaction))
			{
				// check that payment_amount/payment_currency are correct
				// check $payment_amount == $price from $subscription_id && $payment_currency == $price from $subscription_id
				$price 				= $this->_convertNumber($transaction->price);
				$currency 			= $this->normalize($this->params->get('currency', 'amountUSD'));
				$payment_currency 	= $this->normalize($paymentCurrency);
				
				if ((float) $receivedAmount >= (float) $price)
				{
					if ($currency == $payment_currency)
					{
						// set the hash
						$this->setTransactionHash($transaction->id, $hash);
						
						// process payment unless manual activation selected
						$membership_id = $this->getMembershipId($transaction->params, $transaction->type);
						if ($membership_id) {
							$query->clear()
								->select('activation')
								->from($db->qn('#__rsmembership_memberships'))
								->where($db->qn('id').' = '.$db->q((int) $membership_id));
							$db->setQuery($query);
							$activation = $db->loadResult();
							
							if ($activation != MEMBERSHIP_ACTIVATION_MANUAL) {
								RSMembership::approve($transaction->id);
							}
							
							$activationText = 'missing';
							if ($activation == MEMBERSHIP_ACTIVATION_MANUAL) {
								$activationText = 'manual';
							} elseif ($activation == MEMBERSHIP_ACTIVATION_AUTO) {
								$activationText = 'auto';
							} elseif ($activation == MEMBERSHIP_ACTIVATION_INSTANT) {
								$activationText = 'instant';
							}
							
							$log[] = "Activation is $activationText.";
							$log[] = "Successfully added the payment to the database.";
						}
						else {
							$log[] = "The membership could not be found in the database.";
						}
					}
					else
					{
						$log[] = "Expected a currency of $currency. SpicePay reports this payment is made in $payment_currency. Stopping.";
						$deny  = true;
					}
				}
				else
				{
					$log[] = "Expected an amount of $price. SpicePay reports this payment is $receivedAmount. Stopping.";
					$deny  = true;
				}
			}

			//Process callback there 
			echo "OK"; //Say Spicepay server no more callbacks needed
		}
		else
		{
			$log[] = "Expected an invalide hash. Stopping.";
			$deny  = true;
		}

		if ($transaction_id)
		{
			$log[] = "String sent by Spicepay is $hash";
			RSMembership::saveTransactionLog($log, $transaction_id);
			if ($deny) 
				RSMembership::deny($transaction_id);
		}

		$this->finish();
	}
	
	protected function getMembershipId($params, $type) {
		$params = RSMembershipHelper::parseParams($params);
		
		switch ($type)
		{
			case 'new':
			case 'renew':
				$membership_id = $params['membership_id'];
			break;
			case 'addextra':
				JTable::addIncludePath(JPATH_ADMINISTRATOR.'/components/com_rsmembership/tables');
				// Verify if the subscription still exists
				$current = JTable::getInstance('Membership_Subscriber', 'RSMembershipTable');
				if (!$current->load($params['id'])) {
					return false;
				}	
				// Check if this membership still exists
				$membership = JTable::getInstance('Membership', 'RSMembershipTable');
				if (!$membership->load($current->membership_id)) {
					return false;
				}
				
				$membership_id = $current->membership_id;
			
			break;
			case 'upgrade':
				JTable::addIncludePath(JPATH_ADMINISTRATOR.'/components/com_rsmembership/tables');
				$membership = JTable::getInstance('Membership', 'RSMembershipTable');
				if (!$membership->load($params['to_id'])) {
					return false;
				}
				$membership_id = $membership->id;
			break;
		}
		
		return $membership_id;
	}

	protected function _convertNumber($number)
	{
		return number_format($number, 2, '.', '');
	}

	protected function _convertPeriod($period, $type)
	{
		$return = array();
		
		$return[0] = $period;
		$return[1] = strtoupper($type);
		
		return $return;
	}
	protected function _loadLanguage() {
		$this->loadLanguage('plg_system_rsmembership', JPATH_ADMINISTRATOR);
		$this->loadLanguage('plg_system_rsmembershipspicepay', JPATH_ADMINISTRATOR);
	}
	
	protected function escape($string) {
		return htmlentities($string, ENT_COMPAT, 'utf-8');
	}
	
	// Get full transaction details.
	protected function getTransaction($value, $column = 'custom') {
		$db		= JFactory::getDbo();
		$query	= $db->getQuery(true);
		
		$query->select('*')
			  ->from($db->qn('#__rsmembership_transactions'))
			  ->where($db->qn($column).' = '.$db->q($value));
		$db->setQuery($query);
		return $db->loadObject();
	}
	
	// Get transaction ID from custom column.
	protected function getTransactionId($value, $column = 'custom') {
		$db		= JFactory::getDbo();
		$query	= $db->getQuery(true);
		
		$query->select($db->qn('id'))
			  ->from($db->qn('#__rsmembership_transactions'))
			  ->where($db->qn($column).' = '.$db->q($value));
		$db->setQuery($query);
		return (int) $db->loadResult();
	}
	
	// Sets the order number (from the Payment Gateway) to the specified transaction
	protected function setTransactionHash($id, $hash) {
		$db 	= JFactory::getDbo();
		$query 	= $db->getQuery(true);
		
		$query->update($db->qn('#__rsmembership_transactions'))
			  ->set($db->qn('hash').' = '.$db->q($hash))
			  ->where($db->qn('id').' = '.$db->q($id));
		$db->setQuery($query);
		$db->execute();
	}
	
	protected function normalize($string) {
		return strtolower(trim($string));
	}
}