<?php
/**
* @package RSMembership!
* @copyright (C) 2009-2018 www.rsjoomla.com
* @license GPL, http://www.gnu.org/copyleft/gpl.html
*/

// no direct access
defined( '_JEXEC' ) or die( 'Restricted access' );
jimport( 'joomla.plugin.plugin' );

class plgSystemRSMRsmail extends JPlugin
{
	public function __construct(&$subject, $config) {
		parent::__construct($subject, $config);
	}
	
	protected function canRun() 
	{
		$helper1 = JPATH_ADMINISTRATOR.'/components/com_rsmembership/helpers/rsmembership.php';
		$helper2 = JPATH_SITE.'/components/com_rsmail/helpers/rsmail.php';

		if ( file_exists($helper1) && file_exists($helper2) ) {
			$lang = JFactory::getLanguage();
			$lang->load('plg_system_rsmrsmail');
			
			require_once($helper1);
			require_once($helper2);

			return true;
		}

		return false;
	}
	
	function rsm_onAfterMembershipTabs( $vars ) 
	{
		if (!$this->canRun())
		{
			return;
		}

		$tabs = $vars['tabs'];
		$cid  = JFactory::getApplication()->input->get('id', 0, 'int');

		if (file_exists(JPATH_SITE.'/components/com_rsmail/helpers/actions.php'))
		{
			require_once JPATH_SITE.'/components/com_rsmail/helpers/actions.php';
		}

		$row = JTable::getInstance('Rsmail', 'RSMembershipTable');
		if (!$row)
		{
			return;
		}
		
		$row->load($cid);

		$row->rsm_merge_vars = @unserialize($row->rsm_merge_vars);
		if ($row->rsm_merge_vars === false)
		{
			$row->rsm_merge_vars = array();
		}
		
		// Get a new instance of the RSMail! helper
		$rsmail = new rsmHelper();

		$fields_array = $this->_getFields();
		$fields 	  = array();
		$fields[] 	  = JHtml::_('select.option', '', JText::_('RSMEMBERSHIP_RSMAIL_IGNORE'));

		if (!empty($fields_array))
		{
			foreach ($fields_array as $field)
			{
				$fields[] = JHtml::_('select.option', $field->name, $field->label);
			}
		}

		// Merge Vars
		$merge_vars = array();
		if ($row->rsm_list_id)
		{
			$rsm_fields = $rsmail->getFields($row->rsm_list_id);
			if (!empty($rsm_fields))
			{
				foreach ($rsm_fields as $field)
				{
					$merge_vars[$field] = $field;
				}
			}
		}

		$lists['fields'] = array();
		if (is_array($merge_vars))
		{
			foreach ($merge_vars as $merge_var => $title)
			{
				$lists['fields'][$merge_var] = JHtml::_('select.genericlist', $fields, 'rsm_merge_vars['.$merge_var.']', null, 'value', 'text', isset($row->rsm_merge_vars[$merge_var]) ? $row->rsm_merge_vars[$merge_var] : null);
			}
		}
		// RSMail! Lists
		$results 	= $rsmail->getLists();
		$rsm_lists 	= array(
			JHtml::_('select.option', '0', JText::_('RSMEMBERSHIP_RSMAIL_SELECT_LIST'))
		);
		
		if (!empty($results))
		{			
			foreach ($results as $result)
			{
				$rsm_lists[] = JHtml::_('select.option', $result->IdList, $result->ListName);
			}
		}
		
		$lists['rsm_list_id'] 	= JHtml::_('select.genericlist', $rsm_lists, 'rsm_list_id', 'onchange="rsm_changeRsmList(this)"', 'value', 'text', $row->rsm_list_id);
		$lists['published'] 	= JHtml::_('select.booleanlist','rsm_published','class="inputbox"',$row->rsm_published);

		jimport('joomla.filesystem.file');

		// add the tab title
		$tabs->addTitle(JText::_('RSMEMBERSHIP_RSMAIL_INTEGRATION'), 'membership-rsmail');
		
		// add the tab content

		ob_start();
		include JPATH_ADMINISTRATOR.'/components/com_rsmembership/helpers/rsmail.php';
		$content = ob_get_contents();

		$tabs->addContent($content);
		ob_end_clean();
	}

	function rsm_onMembershipSave( $membership ) 
	{
		if (!$this->canRun())
		{
			return;
		}

		$membership = (array) $membership;
		$db 		= JFactory::getDBO();
		$query		= $db->getQuery(true);

		$post['membership_id']  = (int) $membership['id'];
		$post['rsm_merge_vars'] = JFactory::getApplication()->input->get('rsm_merge_vars', array(), 'array');
		$post['rsm_list_id'] 	= JFactory::getApplication()->input->get('rsm_list_id', 0, 'int');
		$post['rsm_published'] 	= JFactory::getApplication()->input->get('rsm_published', 0, 'int');

		$row = JTable::getInstance('Rsmail', 'RSMembershipTable');
		if (!$row)
		{
			return;
		}
		
		if (!$row->bind($post))
		{
			JFactory::getApplication()->enqueueMessage($row->getError(), 'warning');
			return false;
		}

		$row->rsm_merge_vars = serialize($post['rsm_merge_vars']);
		$query->select($db->qn('membership_id'))->from($db->qn('#__rsmembership_rsmail'))->where($db->qn('membership_id').' = '.$db->q((int) $post['membership_id']));
		$db->setQuery($query);
		
		if (!$db->loadResult()) 
		{
			$query->clear();
			$query->insert($db->qn('#__rsmembership_rsmail'))->set($db->qn('membership_id').' = '.$db->q($post['membership_id']));
			$db->setQuery($query);
			$db->execute();
		}
	
		if ($row->store())
		{
			return true;
		} 
		else
		{
			JFactory::getApplication()->enqueueMessage($row->getError(), 'warning');
			return false;
		}
	}

	function rsm_onSwitchTasks() 
	{
		if (!$this->canRun())
		{
			return;
		}
		
		$plugin_task = JFactory::getApplication()->input->get('plugin_task', '', 'string');
		
		if ($plugin_task == 'rsm_get_merge_vars')
		{
			if (file_exists(JPATH_SITE.'/components/com_rsmail/helpers/actions.php'))
			{
				require_once (JPATH_SITE.'/components/com_rsmail/helpers/actions.php');
			}
			
			// Get a new instance of the RSMail! helper
			$rsmail = new rsmHelper();
			// Get list fields
			$results = $rsmail->getFields(JFactory::getApplication()->input->get('list_id', 0, 'int'));

			echo is_array($results) && !empty($results) ? implode("\n",$results) : '';
			jexit();
		}
	}

	function rsm_onSaveRegistration( $args ) 
	{
		if (!$this->canRun())
		{
			return;
		}
		
		$db 		= JFactory::getDBO();
		$query		= $db->getQuery(true);
		$lang 		= JFactory::getLanguage();
		$membership = (int) $args['membership'];
		$email		= $args['email'];
		$data		= $args['data'];
		$userid		= $args['userid'];
		$rsmname	= $data->name;
		
		$lang->load('com_rsmail',JPATH_SITE);

		$query->select('*')->from($db->qn('#__rsmembership_rsmail'))->where($db->qn('membership_id').' = '.$db->q($membership))->where($db->qn('rsm_published').' = '.$db->q('1'));
		$db->setQuery($query);
		
		if ($row = $db->loadObject()) 
		{
			if (empty($email))
			{
				return;
			}
			
			if (!$row->rsm_list_id)
			{
				return;
			}
			
			if (empty($data->fields))
			{
				$data->fields = array();
			}
			
			if (file_exists(JPATH_SITE.'/components/com_rsmail/helpers/actions.php'))
			{
				require_once (JPATH_SITE.'/components/com_rsmail/helpers/actions.php');
			}
			
			// Get a new instance of the RSMail! helper
			$rsmail = new rsmHelper(); 
			
			$row->rsm_merge_vars = @unserialize($row->rsm_merge_vars);
			if ($row->rsm_merge_vars === false)
			{
				$row->rsm_merge_vars = array();
			}
			
			$merge_vars = array();
			foreach ($row->rsm_merge_vars as $tag => $field) 
			{
				if (empty($tag))
				{
					continue;
				}

				if (!isset($data->fields[$field]))
				{
					$data->fields[$field] = '';
				}

				if (is_array($data->fields[$field]))
				{
					array_walk($data->fields[$field], array('plgSystemRSMRsmail', '_escapeCommas'));
					$data->fields[$field] = implode(',', $data->fields[$field]);
				}

				if ($field == 'rsmname')
				{
					$merge_vars[$tag] = $rsmname;
				}
				else
				{
					$merge_vars[$tag] = $data->fields[$field];
				}
			}

			// Prepare list
			$list 	= $rsmail->setList($row->rsm_list_id, $merge_vars);
			$state 	= $rsmail->getState();

			// Subscribe user
			$idsubscriber = $rsmail->subscribe($email, $list, $state, $userid);
			
			if ($idsubscriber)
			{
				// The user must confirm his subscription
				if(!$state)
				{
					$hash = md5($row->rsm_list_id.$idsubscriber.$email);
					$rsmail->confirmation($row->rsm_list_id, $email, $hash);
				}
				
				//send notifications
				$rsmail->notifications($row->rsm_list_id, $email, $merge_vars);
			}
		}
	}
	
	function _getFields() 
	{
		$db 	= JFactory::getDBO();
		$query	= $db->getQuery(true);
		
		
		$name 		 = new stdClass();
		$name->name  = 'rsmname';
		$name->label = JText::_('RSMEMBERSHIP_RSMAIL_NAME');
		
		$query->select($db->qn('name').', '.$db->qn('label'))->from($db->qn('#__rsmembership_fields'))->where($db->qn('published').' = '.$db->q('1'))->order($db->qn('ordering').' ASC');
		$db->setQuery($query);
		$fields = $db->loadObjectList();

		return array_merge(array($name),$fields);
	}
	
	function _escapeCommas(&$item) 
	{
		$item = str_replace(',', '\,', $item);
	}
}