<?php
/**
* @package RSMembership!
* @copyright (C) 2009-2014 www.rsjoomla.com
* @license GPL, http://www.gnu.org/licenses/gpl-2.0.html
*/

defined( '_JEXEC' ) or die( 'Restricted access' );
jimport( 'joomla.plugin.plugin');

class plgRSMembershipZOO extends JPlugin
{
	protected $is_installed;
	
	/*
	* Plugin constructor
	*/
	public function __construct($subject, $config)
	{
		parent::__construct($subject);
		$this->loadLanguage('plg_rsmembership_zoo', JPATH_ADMINISTRATOR);
	}
	
	public function getSupportedSharedTypes()
	{
		return $this->_isZOOInstalled() ? array('zooarticle', 'zoocategory', 'zooapplication') : array();
	}

	protected function _isZOOInstalled()
	{
		if (!is_bool($this->is_installed))
		{
			$this->is_installed = file_exists(JPATH_ADMINISTRATOR.'/components/com_zoo/zoo.php');
		}

		return $this->is_installed;
	}
	
	protected function _isSupportedSharedType($shared_type)
	{
		return in_array($shared_type, $this->getSupportedSharedTypes());
	}
	
	public function showUserFriendlyParams(&$shared)
	{
		if (!$this->_isSupportedSharedType($shared->type))
		{
			return;
		}
		
		$db 	= JFactory::getDBO();
		$query	= $db->getQuery(true);
		
		switch ($shared->type)
		{
			case 'zooarticle':
				$query->select($db->qn('i.name','item_name').', '.$db->qn('a.name','app_name'))
					  ->from($db->qn('#__zoo_item', 'i'))
					  ->join('left', $db->qn('#__zoo_application', 'a').' ON '.$db->qn('a.id').' = '.$db->qn('i.application_id'))
					  ->where($db->qn('i.id').' = '.$db->q($shared->params));
				$db->setQuery($query);
				$result = $db->loadObject();

				if ($result)
				{
					$shared->params = "({$result->app_name}) {$result->item_name}";
				}
			break;

			case 'zoocategory':
				$query->select($db->qn('c.name', 'item_name').', '.$db->qn('a.name', 'app_name'))
					  ->from($db->qn('#__zoo_category', 'c'))
					  ->join('left', $db->qn('#__zoo_application', 'a').' ON '.$db->qn('a.id').' = '.$db->qn('c.application_id'))
					  ->where($db->qn('c.id').' = '.$db->q($shared->params));
				$db->setQuery($query);
				$result = $db->loadObject();

				if ($result)
				{
					$shared->params = "({$result->app_name}) {$result->item_name}";
				}
			break;

			case 'zooapplication':
				$query->select($db->qn('a.name'))->from($db->qn('#__zoo_application', 'a'))->where($db->qn('a.id').' = '.$db->q($shared->params));
				$db->setQuery($query);
				$result = $db->loadResult();

				if ($result)
				{
					$shared->params = $result;
				}

			break;
		}
	}

	public function getData($shared_type, &$data, $limitstart=0, $limit=20)
	{
		if (!$this->_isSupportedSharedType($shared_type))
		{
			return;
		}

		$db 	= JFactory::getDBO();
		$query	= $db->getQuery(true);

		$this->getSortColumn($shared_type, $sortColumn);
		$this->getSortOrder($shared_type, $sortOrder);

		$filter_word = $this->getFilterWord();

		switch ($shared_type)
		{
			case 'zooarticle':
				$query->select($db->qn('i.name').', '.$db->qn('a.name', 'app_name').', '.$db->qn('i.id'))
					  ->from($db->qn('#__zoo_item', 'i'))
					  ->join('left', $db->qn('#__zoo_application', 'a').' ON '.$db->qn('i.application_id').' = '.$db->qn('a.id'))
					  ->order($db->qn($sortColumn).' '.$sortOrder);
	  
				if ( $filter_word )
				{
					$query->where($db->qn('i.name').' LIKE '.$db->q('%'.$filter_word.'%'));
				}

				$db->setQuery($query, $limitstart, $limit);
				$data = $db->loadObjectList();
			break;

			case 'zoocategory':
				$query->select($db->qn('c.name').', '.$db->qn('a.name', 'app_name').', '.$db->qn('c.id'))
					  ->from($db->qn('#__zoo_category', 'c'))
					  ->join('left', $db->qn('#__zoo_application', 'a').' ON '.$db->qn('c.application_id').' = '.$db->qn('a.id'))
					  ->order($db->qn($sortColumn).' '.$sortOrder);
					  
				if ( $filter_word )
				{
					$query->where($db->qn('c.name').' LIKE '.$db->q('%'.$filter_word.'%'));
				}
				
				$db->setQuery($query, $limitstart, $limit);
				$data = $db->loadObjectList();
			break;

			case 'zooapplication':
				$query->select($db->qn('a.name').', '.$db->qn('a.id'))->from($db->qn('#__zoo_application', 'a'))->order($db->qn($sortColumn).' '.$sortOrder);
				
				if ( $filter_word )
				{
					$query->where($db->qn('a.name').' LIKE '.$db->q('%'.$filter_word.'%'));
				}

				$db->setQuery($query, $limitstart, $limit);
				$data = $db->loadObjectList();
			break;
		}
	}
	
	public function getTotal($shared_type, &$total)
	{
		if (!$this->_isSupportedSharedType($shared_type))
		{
			return;
		}
		
		$db 	= JFactory::getDBO();
		$query	= $db->getQuery(true);
		
		$filter_word = $this->getFilterWord();

		switch ($shared_type)
		{
			case 'zooarticle':
				$query->select('COUNT('.$db->qn('i.id').')')->from($db->qn('#__zoo_item', 'i'));
				if ( $filter_word )
				{
					$query->where($db->qn('i.name').' LIKE '.$db->q('%'.$filter_word.'%'));
				}

				$db->setQuery($query);
				$total = $db->loadResult();
			break;

			case 'zoocategory':
				$query->select('COUNT('.$db->qn('c.id').')')->from($db->qn('#__zoo_category','c'));
				if ( $filter_word )
				{
					$query->where($db->qn('c.name').' LIKE '.$db->q('%'.$filter_word.'%'));
				}

				$db->setQuery($query);
				$total = $db->loadResult();
			break;

			case 'zooapplication':
				$query->select('COUNT('.$db->qn('a.id').')')->from($db->qn('#__zoo_application', 'a'));
				if ( $filter_word )
				{
					$query->where($db->qn('a.name').' LIKE '.$db->q('%'.$filter_word.'%'));
				}

				$db->setQuery($query);
				$total = $db->loadResult();
			break;
		}
	}

	public function getSortColumn($shared_type, &$sortColumn)
	{
		if (!$this->_isSupportedSharedType($shared_type))
		{
			return;
		}
		
		$jinput = JFactory::getApplication()->input;
		
		switch ($shared_type)
		{
			case 'zooarticle':
				$sortColumn = $jinput->get('filter_order', 'i.name', 'string');
			break;
			
			case 'zoocategory':
				$sortColumn = $jinput->get('filter_order', 'c.name', 'string');
			break;
			
			case 'zooapplication':
				$sortColumn = $jinput->get('filter_order', 'a.name', 'string');
			break;
		}
	}
	
	public function getSortOrder($shared_type, &$sortOrder)
	{
		$sortOrder = JFactory::getApplication()->input->get('filter_order_Dir', 'ASC', 'string');
	}
	
	public function getFilterWord()
	{
		return JFactory::getApplication()->input->get('search', '', 'string');
	}
	
	public function getHeaders($shared_type, &$headers)
	{
		if (!$this->_isSupportedSharedType($shared_type))
		{
			return;
		}
		
		$this->getSortColumn($shared_type, $sortColumn);
		$this->getSortOrder($shared_type, $sortOrder);
		
		switch ($shared_type)
		{
			case 'zooarticle':
				$headers = array(			
					array('attributes' => '', 'innerHTML' => JHtml::_('grid.sort', 'TITLE', 'i.name', $sortOrder, $sortColumn)),
					array('attributes' => '', 'innerHTML' => JHtml::_('grid.sort', 'PLG_RSMEMBERSHIP_TYPE_APPLICATION', 'a.name', $sortOrder, $sortColumn)),
					array('attributes' => 'width="5"', 'innerHTML' => JHtml::_('grid.sort', 'ID', 'i.id', $sortOrder, $sortColumn))
				);
			break;
			
			case 'zoocategory':
				$headers = array(			
					array('attributes' => '', 'innerHTML' => JHtml::_('grid.sort', 'TITLE', 'c.name', $sortOrder, $sortColumn)),
					array('attributes' => '', 'innerHTML' => JHtml::_('grid.sort', 'PLG_RSMEMBERSHIP_TYPE_APPLICATION', 'a.name', $sortOrder, $sortColumn)),
					array('attributes' => 'width="5"', 'innerHTML' => JHtml::_('grid.sort', 'ID', 'c.id', $sortOrder, $sortColumn))
				);
			break;
			
			case 'zooapplication':
				$headers = array(			
					array('attributes' => '', 'innerHTML' => JHtml::_('grid.sort', 'PLG_RSMEMBERSHIP_TYPE_APPLICATION', 'a.name', $sortOrder, $sortColumn)),
					array('attributes' => 'width="5"', 'innerHTML' => JHtml::_('grid.sort', 'ID', 'a.id', $sortOrder, $sortColumn))
				);
			break;
		}
	}

	protected function _getCategoryParents($catid)
	{
		$db 		= JFactory::getDBO();
		$query		= $db->getQuery(true);
		$categories = array();

		$query->select($db->qn('parent'))->from($db->qn('#__zoo_category'))->where($db->qn('id').' = '.$db->q($catid));
		$db->setQuery($query);
		while ( $catid = $db->loadResult() ) 
		{
			$categories[] = $catid;
			
			$query->clear();
			$query->select($db->qn('parent'))->from($db->qn('#__zoo_category'))->where($db->qn('id').' = '.$db->q($catid));
			$db->setQuery($query);
		}

		return $categories;
	}
	
	protected function _getItemCategories($item_id)
	{
		// Get the database object
		$db 	= JFactory::getDBO();
		$query	= $db->getQuery(true);
		

		$query->select($db->qn('category_id'))->from($db->qn('#__zoo_category_item'))->where($db->qn('item_id').' = '.$db->q($item_id));
		$db->setQuery($query);
		$category_id = $db->loadResult();
		$categories = $this->_getCategoryParents($category_id);
		array_push($categories, $category_id);

		return $categories;
	}
	
	protected function _getItemApplication($item_id)
	{
		// Get the database object
		$db 	= JFactory::getDBO();
		$query	= $db->getQuery(true);
		
		$query->select($db->qn('application_id'))->from($db->qn('#__zoo_item'))->where($db->qn('id').' = '.$db->q($item_id));
		$db->setQuery($query);

		return $db->loadResult();
	}

	public function checkShared($option, &$has_access, &$found_shared, &$redirect, $memberships, $extras) 
	{
		// Runs only in frontend
		$app 	= JFactory::getApplication();
		$jinput = $app->input;
		
		if ($app->isAdmin())
		{
			return;
		}
		// Runs only with com_zoo
		if ($option != 'com_zoo')
		{
			return;
		}
		// Performance check
		if ($has_access)
		{
			return;
		}
		
		// Get the database object
		$db 	= JFactory::getDBO();
		$query	= $db->getQuery(true);

		$shared = RSMembershipHelper::getShared($this->getSupportedSharedTypes());

		$view 	 = $jinput->get('view', '', 'cmd');
		$task 	 = $jinput->get('task', '', 'cmd');
		$item_id = $jinput->get('item_id', '', 'int');
		$Itemid  = $jinput->get('Itemid', 0, 'int');

		if ($view == 'item' || $task == 'item') 
		{
			if (!$item_id && $Itemid > 0)
			{
				$menu 	 = JSite::getMenu();
				$item 	 = $menu->getItem($Itemid);
				$params  = new JRegistry();
				$params->loadString($item->params);
				$params  = $params->toArray();

				$item_id = $params['item_id'];
			}

			// Must check if it belongs to a category that we are sharing
			$categories = $this->_getItemCategories($item_id);
			
			// Must check if it belongs to an application that we are sharing
			$application_id = $this->_getItemApplication($item_id);
			
			foreach ($shared as $share)
			{
				$what  = isset($share->membership_id) ? 'membership_id' : 'extra_value_id';
				$where = isset($share->membership_id) ? $memberships : $extras;
				$table = isset($share->membership_id) ? '#__rsmembership_memberships' : '#__rsmembership_extra_values';
					
				if (($share->type == 'zooarticle' && $share->params == $item_id)
					|| ($share->type == 'zoocategory' && in_array($share->params, $categories))
					|| ($share->type == 'zooapplication' && $share->params == $application_id))
				{
					$found_shared = true;
					
					// Found a membership that shares this article
					if (!empty($where) && in_array($share->{$what}, $where))
					{
						$has_access = true;
						break;
					}
					else
					{
						// Get the redirect page
						$query->clear();
						$query->select($db->qn('share_redirect'))->from($db->qn($table))->where($db->qn('id').' = '.$db->q($share->{$what}));
						$db->setQuery($query);
						$redirect = $db->loadResult();
					}
				}
			}
		}
		elseif ($view == 'category' || $task == 'category')
		{
			if (!$item_id && $jinput->get('Itemid', 0, 'int') > 0)
			{
				$item 	= $app->getMenu()->getItem($jinput->get('Itemid', 0, 'int'));
				$params = $item->params;
				$item_id = $params->get('category');
			}
			
			// Must check if it belongs to a parent that we are sharing
			$categories = $this->_getCategoryParents($item_id);
			array_push($categories, $item_id);
			
			// Must check if it belongs to an application that we are sharing
			$application_id = $this->_getItemApplication($item_id);

			foreach ($shared as $share)
			{
				$what  = isset($share->membership_id) ? 'membership_id' : 'extra_value_id';
				$where = isset($share->membership_id) ? $memberships : $extras;
				$table = isset($share->membership_id) ? '#__rsmembership_memberships' : '#__rsmembership_extra_values';

				if ( $share->type == 'zoocategory' && in_array($share->params, $categories) || ($share->type == 'zooapplication' && $share->params == $application_id) ) 
				{
					$found_shared = true;
					// Found a membership that shares this category
					if (!empty($where) && in_array($share->{$what}, $where))
					{
						$has_access = true;
						break;
					}
					else
					{
						// Get the redirect page
						$query->clear();
						$query->select($db->qn('share_redirect'))->from($db->qn($table))->where($db->qn('id').' = '.$db->q($share->{$what}));
						$db->setQuery($query);
						$redirect = $db->loadResult();
					}
				}
			}
		}
		elseif ($view == 'frontpage')
		{
			$item 	= $app->getMenu()->getItem($jinput->get('Itemid', 0, 'int'));
			$params = $item->params;
			$application_id = $params->get('application');

			foreach ( $shared as $share ) 
			{
				$what  = isset($share->membership_id) ? 'membership_id' : 'extra_value_id';
				$where = isset($share->membership_id) ? $memberships : $extras;
				$table = isset($share->membership_id) ? '#__rsmembership_memberships' : '#__rsmembership_extra_values';
					
				if ( $share->type == 'zooapplication' && $share->params == $application_id )
				{
					$found_shared = true;

					// Found a membership that shares this category
					if (!empty($where) && in_array($share->{$what}, $where))
					{
						$has_access = true;
						break;
					}
					else
					{
						$query->clear();
						$query->select($db->qn('share_redirect'))->from($db->qn($table))->where($db->qn('id').' = '.$db->q($share->{$what}));
						$db->setQuery($query);
						$redirect = $db->loadResult();
					}
				}
			}
		}
	}
}